@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,36 +28,30 @@
 #define phpext_mongodb_ptr &mongodb_module_entry
 extern zend_module_entry mongodb_module_entry;
 
-/* Structure for persisted libmongoc clients. The PID is included to ensure that
- * processes do not destroy clients created by other processes (relevant for
- * forking). We avoid using pid_t for Windows compatibility. */
+/* Structure for tracking libmongoc clients (both persisted and non-persisted).
+ * The PID is included to ensure that processes do not destroy clients created
+ * by other processes (relevant for forking). We avoid using pid_t for Windows
+ * compatibility. */
 typedef struct {
 	mongoc_client_t* client;
-	int              pid;
+	int              created_by_pid;
+	int              last_reset_by_pid;
+	bool             is_persistent;
 } php_phongo_pclient_t;
 
 ZEND_BEGIN_MODULE_GLOBALS(mongodb)
-	char*             debug;
-	FILE*             debug_fd;
-	bson_mem_vtable_t bsonMemVTable;
-	HashTable         pclients;
-	HashTable*        subscribers;
+	char*      debug;
+	FILE*      debug_fd;
+	HashTable  persistent_clients;
+	HashTable* request_clients;
+	HashTable* subscribers;
+	HashTable* managers;
 ZEND_END_MODULE_GLOBALS(mongodb)
 
-#if PHP_VERSION_ID >= 70000
 #define MONGODB_G(v) ZEND_MODULE_GLOBALS_ACCESSOR(mongodb, v)
 #if defined(ZTS) && defined(COMPILE_DL_MONGODB)
 ZEND_TSRMLS_CACHE_EXTERN()
 #endif
-#else /* PHP_VERSION_ID >= 70000 */
-#ifdef ZTS
-#define MONGODB_G(v) TSRMG(mongodb_globals_id, zend_mongodb_globals*, v)
-#define mglo mongodb_globals_id
-#else /* ZTS */
-#define MONGODB_G(v) (mongodb_globals.v)
-#define mglo mongodb_globals
-#endif /* ZTS */
-#endif /* PHP_VERSION_ID >= 70000 */
 
 #define PHONGO_WRITE_CONCERN_W_MAJORITY "majority"
 
@@ -99,19 +93,9 @@ typedef enum {
 
 zend_class_entry* phongo_exception_from_mongoc_domain(uint32_t /* mongoc_error_domain_t */ domain, uint32_t /* mongoc_error_code_t */ code);
 zend_class_entry* phongo_exception_from_phongo_domain(php_phongo_error_domain_t domain);
-void              phongo_throw_exception(php_phongo_error_domain_t domain TSRMLS_DC, const char* format, ...)
-#if PHP_VERSION_ID < 70000
-#ifndef PHP_WIN32
-#ifdef ZTS
-	__attribute__((format(printf, 3, 4)))
-#else
-	__attribute__((format(printf, 2, 3)))
-#endif /* ZTS */
-#endif /* PHP_WIN32 */
-#endif /* PHP_VERSION_ID < 70000 */
-	;
-void phongo_throw_exception_from_bson_error_t(bson_error_t* error TSRMLS_DC);
-void phongo_throw_exception_from_bson_error_t_and_reply(bson_error_t* error, const bson_t* reply TSRMLS_DC);
+void              phongo_throw_exception(php_phongo_error_domain_t domain, const char* format, ...);
+void              phongo_throw_exception_from_bson_error_t(bson_error_t* error);
+void              phongo_throw_exception_from_bson_error_t_and_reply(bson_error_t* error, const bson_t* reply);
 
 /* This enum is used for processing options in phongo_execute_parse_options and
  * selecting a libmongoc function to use in phongo_execute_command. The values
@@ -129,65 +113,61 @@ typedef enum {
 
 zend_object_handlers* phongo_get_std_object_handlers(void);
 
-void phongo_server_init(zval* return_value, mongoc_client_t* client, uint32_t server_id TSRMLS_DC);
-void phongo_session_init(zval* return_value, mongoc_client_session_t* client_session TSRMLS_DC);
-void phongo_readconcern_init(zval* return_value, const mongoc_read_concern_t* read_concern TSRMLS_DC);
-void phongo_readpreference_init(zval* return_value, const mongoc_read_prefs_t* read_prefs TSRMLS_DC);
-void phongo_writeconcern_init(zval* return_value, const mongoc_write_concern_t* write_concern TSRMLS_DC);
-bool phongo_execute_bulk_write(mongoc_client_t* client, const char* namespace, php_phongo_bulkwrite_t* bulk_write, zval* zwriteConcern, uint32_t server_id, zval* return_value, int return_value_used TSRMLS_DC);
-bool phongo_execute_command(mongoc_client_t* client, php_phongo_command_type_t type, const char* db, zval* zcommand, zval* zreadPreference, uint32_t server_id, zval* return_value, int return_value_used TSRMLS_DC);
-bool phongo_execute_query(mongoc_client_t* client, const char* namespace, zval* zquery, zval* zreadPreference, uint32_t server_id, zval* return_value, int return_value_used TSRMLS_DC);
+void phongo_clientencryption_init(php_phongo_clientencryption_t* ce_obj, zval* manager, zval* options);
+void phongo_server_init(zval* return_value, zval* manager, uint32_t server_id);
+void phongo_session_init(zval* return_value, zval* manager, mongoc_client_session_t* client_session);
+void phongo_readconcern_init(zval* return_value, const mongoc_read_concern_t* read_concern);
+void phongo_readpreference_init(zval* return_value, const mongoc_read_prefs_t* read_prefs);
+void phongo_writeconcern_init(zval* return_value, const mongoc_write_concern_t* write_concern);
+bool phongo_execute_bulk_write(zval* manager, const char* namespace, php_phongo_bulkwrite_t* bulk_write, zval* zwriteConcern, uint32_t server_id, zval* return_value);
+bool phongo_execute_command(zval* manager, php_phongo_command_type_t type, const char* db, zval* zcommand, zval* zreadPreference, uint32_t server_id, zval* return_value);
+bool phongo_execute_query(zval* manager, const char* namespace, zval* zquery, zval* zreadPreference, uint32_t server_id, zval* return_value);
 
-bool phongo_cursor_advance_and_check_for_error(mongoc_cursor_t* cursor TSRMLS_DC);
+bool phongo_cursor_advance_and_check_for_error(mongoc_cursor_t* cursor);
 
-const mongoc_read_concern_t*  phongo_read_concern_from_zval(zval* zread_concern TSRMLS_DC);
-const mongoc_read_prefs_t*    phongo_read_preference_from_zval(zval* zread_preference TSRMLS_DC);
-const mongoc_write_concern_t* phongo_write_concern_from_zval(zval* zwrite_concern TSRMLS_DC);
+const mongoc_read_concern_t*  phongo_read_concern_from_zval(zval* zread_concern);
+const mongoc_read_prefs_t*    phongo_read_preference_from_zval(zval* zread_preference);
+const mongoc_write_concern_t* phongo_write_concern_from_zval(zval* zwrite_concern);
 
 php_phongo_server_description_type_t php_phongo_server_description_type(mongoc_server_description_t* sd);
 
-bool phongo_parse_read_preference(zval* options, zval** zreadPreference TSRMLS_DC);
+bool phongo_parse_read_preference(zval* options, zval** zreadPreference);
+bool phongo_parse_session(zval* options, mongoc_client_t* client, bson_t* mongoc_opts, zval** zsession);
 
-zval* php_phongo_prep_legacy_option(zval* options, const char* key, bool* allocated TSRMLS_DC);
-void  php_phongo_prep_legacy_option_free(zval* options TSRMLS_DC);
+zval* php_phongo_prep_legacy_option(zval* options, const char* key, bool* allocated);
+void  php_phongo_prep_legacy_option_free(zval* options);
 
-void php_phongo_read_preference_prep_tagsets(zval* tagSets TSRMLS_DC);
+void php_phongo_read_preference_prep_tagsets(zval* tagSets);
 bool php_phongo_read_preference_tags_are_valid(const bson_t* tags);
 
-void php_phongo_server_to_zval(zval* retval, mongoc_server_description_t* sd);
+bool php_phongo_server_to_zval(zval* retval, mongoc_server_description_t* sd);
 void php_phongo_read_concern_to_zval(zval* retval, const mongoc_read_concern_t* read_concern);
 void php_phongo_write_concern_to_zval(zval* retval, const mongoc_write_concern_t* write_concern);
-void php_phongo_cursor_to_zval(zval* retval, const mongoc_cursor_t* cursor);
 
-void phongo_manager_init(php_phongo_manager_t* manager, const char* uri_string, zval* options, zval* driverOptions TSRMLS_DC);
-int  php_phongo_set_monitoring_callbacks(mongoc_client_t* client);
-void php_phongo_objectid_new_from_oid(zval* object, const bson_oid_t* oid TSRMLS_DC);
-void php_phongo_cursor_id_new_from_id(zval* object, int64_t cursorid TSRMLS_DC);
-void php_phongo_new_utcdatetime_from_epoch(zval* object, int64_t msec_since_epoch TSRMLS_DC);
-void php_phongo_new_timestamp_from_increment_and_timestamp(zval* object, uint32_t increment, uint32_t timestamp TSRMLS_DC);
-void php_phongo_new_javascript_from_javascript(int init, zval* object, const char* code, size_t code_len TSRMLS_DC);
-void php_phongo_new_javascript_from_javascript_and_scope(int init, zval* object, const char* code, size_t code_len, const bson_t* scope TSRMLS_DC);
-void php_phongo_new_binary_from_binary_and_type(zval* object, const char* data, size_t data_len, bson_subtype_t type TSRMLS_DC);
-void php_phongo_new_decimal128(zval* object, const bson_decimal128_t* decimal TSRMLS_DC);
-void php_phongo_new_int64(zval* object, int64_t integer TSRMLS_DC);
-void php_phongo_new_regex_from_regex_and_options(zval* object, const char* pattern, const char* flags TSRMLS_DC);
-void php_phongo_new_symbol(zval* object, const char* symbol, size_t symbol_len TSRMLS_DC);
-void php_phongo_new_dbpointer(zval* object, const char* namespace, size_t namespace_len, const bson_oid_t* oid TSRMLS_DC);
+void phongo_manager_init(php_phongo_manager_t* manager, const char* uri_string, zval* options, zval* driverOptions);
+bool php_phongo_set_monitoring_callbacks(mongoc_client_t* client);
 
-zend_bool phongo_writeerror_init(zval* return_value, bson_t* bson TSRMLS_DC);
-zend_bool phongo_writeconcernerror_init(zval* return_value, bson_t* bson TSRMLS_DC);
+bool php_phongo_parse_int64(int64_t* retval, const char* data, size_t data_len);
+
+void phongo_clientencryption_create_datakey(php_phongo_clientencryption_t* clientencryption, zval* return_value, char* kms_provider, zval* options);
+void phongo_clientencryption_encrypt(php_phongo_clientencryption_t* clientencryption, zval* zvalue, zval* zciphertext, zval* options);
+void phongo_clientencryption_decrypt(php_phongo_clientencryption_t* clientencryption, zval* zciphertext, zval* zvalue);
+
+zend_bool phongo_writeerror_init(zval* return_value, bson_t* bson);
+zend_bool phongo_writeconcernerror_init(zval* return_value, bson_t* bson);
+
+void php_phongo_client_reset_once(php_phongo_manager_t* manager, int pid);
+
+bool php_phongo_client_register(php_phongo_manager_t* manager);
+bool php_phongo_client_unregister(php_phongo_manager_t* manager);
+
+bool php_phongo_manager_register(php_phongo_manager_t* manager);
+bool php_phongo_manager_unregister(php_phongo_manager_t* manager);
 
-#if PHP_VERSION_ID >= 70000
 #define PHONGO_CE_FINAL(ce)             \
 	do {                                \
 		ce->ce_flags |= ZEND_ACC_FINAL; \
 	} while (0);
-#else
-#define PHONGO_CE_FINAL(ce)                   \
-	do {                                      \
-		ce->ce_flags |= ZEND_ACC_FINAL_CLASS; \
-	} while (0);
-#endif
 
 #define PHONGO_CE_DISABLE_SERIALIZATION(ce)            \
 	do {                                               \
@@ -209,9 +189,32 @@ zend_bool phongo_writeconcernerror_init(zval* return_value, bson_t* bson TSRMLS_
 		}                                                                  \
 	} while (0);
 
+#define PHONGO_GET_PROPERTY_HASH_FREE_PROPS(is_debug, props) \
+	do {                                                     \
+		if (is_debug) {                                      \
+			zend_hash_destroy((props));                      \
+			FREE_HASHTABLE(props);                           \
+		}                                                    \
+	} while (0);
+
 #define PHONGO_ZVAL_CLASS_OR_TYPE_NAME(zv) (Z_TYPE(zv) == IS_OBJECT ? ZSTR_VAL(Z_OBJCE(zv)->name) : zend_get_type_by_const(Z_TYPE(zv)))
 #define PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(zvp) PHONGO_ZVAL_CLASS_OR_TYPE_NAME(*(zvp))
 
+#define PHONGO_ZVAL_EXCEPTION_NAME(e) (ZSTR_VAL(e->ce->name))
+
+#define PHONGO_SET_CREATED_BY_PID(intern)          \
+	do {                                           \
+		(intern)->created_by_pid = (int) getpid(); \
+	} while (0);
+
+#define PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, manager) \
+	do {                                                    \
+		int pid = (int) getpid();                           \
+		if ((intern)->created_by_pid != pid) {              \
+			php_phongo_client_reset_once((manager), pid);   \
+		}                                                   \
+	} while (0);
+
 #endif /* PHONGO_H */
 
 /*
