@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -73,15 +73,15 @@ static bool php_phongo_binary_init_from_hash(php_phongo_binary_t* intern, HashTa
    Construct a new BSON binary type */
 static PHP_METHOD(Binary, __construct)
 {
-	php_phongo_binary_t* intern;
 	zend_error_handling  error_handling;
+	php_phongo_binary_t* intern;
 	char*                data;
 	size_t               data_len;
 	zend_long            type;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_BINARY_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sl", &data, &data_len, &type) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -91,17 +91,21 @@ static PHP_METHOD(Binary, __construct)
 	php_phongo_binary_init(intern, data, data_len, type);
 } /* }}} */
 
-/* {{{ proto void MongoDB\BSON\Binary::__set_state(array $properties)
+/* {{{ proto MongoDB\BSON\Binary MongoDB\BSON\Binary::__set_state(array $properties)
 */
 static PHP_METHOD(Binary, __set_state)
 {
+	zend_error_handling  error_handling;
 	php_phongo_binary_t* intern;
 	HashTable*           props;
 	zval*                array;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_binary_ce);
 
@@ -115,11 +119,15 @@ static PHP_METHOD(Binary, __set_state)
    Return the Binary's data string. */
 static PHP_METHOD(Binary, __toString)
 {
+	zend_error_handling  error_handling;
 	php_phongo_binary_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_BINARY_OBJ_P(getThis());
 
@@ -130,13 +138,17 @@ static PHP_METHOD(Binary, __toString)
 */
 static PHP_METHOD(Binary, getData)
 {
+	zend_error_handling  error_handling;
 	php_phongo_binary_t* intern;
 
 	intern = Z_BINARY_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_STRINGL(intern->data, intern->data_len);
 } /* }}} */
@@ -145,13 +157,17 @@ static PHP_METHOD(Binary, getData)
 */
 static PHP_METHOD(Binary, getType)
 {
+	zend_error_handling  error_handling;
 	php_phongo_binary_t* intern;
 
 	intern = Z_BINARY_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_LONG(intern->type);
 } /* }}} */
@@ -160,13 +176,17 @@ static PHP_METHOD(Binary, getType)
 */
 static PHP_METHOD(Binary, jsonSerialize)
 {
+	zend_error_handling  error_handling;
 	php_phongo_binary_t* intern;
 	char                 type[3];
 	int                  type_len;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_BINARY_OBJ_P(getThis());
 
@@ -186,6 +206,7 @@ static PHP_METHOD(Binary, jsonSerialize)
 */
 static PHP_METHOD(Binary, serialize)
 {
+	zend_error_handling  error_handling;
 	php_phongo_binary_t* intern;
 	zval                 retval;
 	php_serialize_data_t var_hash;
@@ -193,9 +214,12 @@ static PHP_METHOD(Binary, serialize)
 
 	intern = Z_BINARY_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	array_init_size(&retval, 2);
 	ADD_ASSOC_STRINGL(&retval, "data", intern->data, intern->data_len);
@@ -216,8 +240,8 @@ static PHP_METHOD(Binary, serialize)
 */
 static PHP_METHOD(Binary, unserialize)
 {
-	php_phongo_binary_t*   intern;
 	zend_error_handling    error_handling;
+	php_phongo_binary_t*   intern;
 	char*                  serialized;
 	size_t                 serialized_len;
 	zval                   props;
@@ -226,7 +250,6 @@ static PHP_METHOD(Binary, unserialize)
 	intern = Z_BINARY_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -312,14 +335,14 @@ static zend_object* php_phongo_binary_create_object(zend_class_entry* class_type
 	return &intern->std;
 } /* }}} */
 
-static zend_object* php_phongo_binary_clone_object(zval* object) /* {{{ */
+static zend_object* php_phongo_binary_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	php_phongo_binary_t* intern;
 	php_phongo_binary_t* new_intern;
 	zend_object*         new_object;
 
-	intern     = Z_BINARY_OBJ_P(object);
-	new_object = php_phongo_binary_create_object(Z_OBJCE_P(object));
+	intern     = Z_OBJ_BINARY(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_binary_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
 	new_intern = Z_OBJ_BINARY(new_object);
 	zend_objects_clone_members(&new_intern->std, &intern->std);
@@ -333,6 +356,8 @@ static int php_phongo_binary_compare_objects(zval* o1, zval* o2) /* {{{ */
 {
 	php_phongo_binary_t *intern1, *intern2;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_BINARY_OBJ_P(o1);
 	intern2 = Z_BINARY_OBJ_P(o2);
 
@@ -349,12 +374,12 @@ static int php_phongo_binary_compare_objects(zval* o1, zval* o2) /* {{{ */
 	return zend_binary_strcmp(intern1->data, intern1->data_len, intern2->data, intern2->data_len);
 } /* }}} */
 
-static HashTable* php_phongo_binary_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+static HashTable* php_phongo_binary_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_binary_t* intern;
 	HashTable*           props;
 
-	intern = Z_BINARY_OBJ_P(object);
+	intern = Z_OBJ_BINARY(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
 
@@ -375,13 +400,13 @@ static HashTable* php_phongo_binary_get_properties_hash(zval* object, bool is_de
 	return props;
 } /* }}} */
 
-static HashTable* php_phongo_binary_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_binary_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_binary_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_binary_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_binary_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_binary_get_properties_hash(object, false);
 } /* }}} */
@@ -402,12 +427,12 @@ void php_phongo_binary_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_binary_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_binary, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_binary.clone_obj       = php_phongo_binary_clone_object;
-	php_phongo_handler_binary.compare_objects = php_phongo_binary_compare_objects;
-	php_phongo_handler_binary.get_debug_info  = php_phongo_binary_get_debug_info;
-	php_phongo_handler_binary.get_properties  = php_phongo_binary_get_properties;
-	php_phongo_handler_binary.free_obj        = php_phongo_binary_free_object;
-	php_phongo_handler_binary.offset          = XtOffsetOf(php_phongo_binary_t, std);
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(binary);
+	php_phongo_handler_binary.clone_obj      = php_phongo_binary_clone_object;
+	php_phongo_handler_binary.get_debug_info = php_phongo_binary_get_debug_info;
+	php_phongo_handler_binary.get_properties = php_phongo_binary_get_properties;
+	php_phongo_handler_binary.free_obj       = php_phongo_binary_free_object;
+	php_phongo_handler_binary.offset         = XtOffsetOf(php_phongo_binary_t, std);
 
 	zend_declare_class_constant_long(php_phongo_binary_ce, ZEND_STRL("TYPE_GENERIC"), BSON_SUBTYPE_BINARY);
 	zend_declare_class_constant_long(php_phongo_binary_ce, ZEND_STRL("TYPE_FUNCTION"), BSON_SUBTYPE_FUNCTION);
