@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -46,7 +46,8 @@ static bool php_phongo_dbpointer_init(php_phongo_dbpointer_t* intern, const char
 	intern->ref     = estrndup(ref, ref_len);
 	intern->ref_len = ref_len;
 
-	strncpy(intern->id, id, sizeof(intern->id));
+	memset(intern->id, 0, sizeof(intern->id));
+	strncpy(intern->id, id, sizeof(intern->id) - 1);
 
 	return true;
 } /* }}} */
@@ -71,13 +72,17 @@ static bool php_phongo_dbpointer_init_from_hash(php_phongo_dbpointer_t* intern,
    Return the DBPointer's namespace string and ObjectId. */
 static PHP_METHOD(DBPointer, __toString)
 {
+	zend_error_handling     error_handling;
 	php_phongo_dbpointer_t* intern;
 	char*                   retval;
 	int                     retval_len;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
@@ -90,13 +95,17 @@ static PHP_METHOD(DBPointer, __toString)
 */
 static PHP_METHOD(DBPointer, jsonSerialize)
 {
+	zend_error_handling     error_handling;
 	php_phongo_dbpointer_t* intern;
 	zval                    zdb_pointer;
 	zval                    zoid;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
@@ -114,6 +123,7 @@ static PHP_METHOD(DBPointer, jsonSerialize)
 */
 static PHP_METHOD(DBPointer, serialize)
 {
+	zend_error_handling     error_handling;
 	php_phongo_dbpointer_t* intern;
 	zval                    retval;
 	php_serialize_data_t    var_hash;
@@ -121,9 +131,12 @@ static PHP_METHOD(DBPointer, serialize)
 
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	array_init_size(&retval, 2);
 	ADD_ASSOC_STRINGL(&retval, "ref", intern->ref, intern->ref_len);
@@ -144,8 +157,8 @@ static PHP_METHOD(DBPointer, serialize)
 */
 static PHP_METHOD(DBPointer, unserialize)
 {
-	php_phongo_dbpointer_t* intern;
 	zend_error_handling     error_handling;
+	php_phongo_dbpointer_t* intern;
 	char*                   serialized;
 	size_t                  serialized_len;
 	zval                    props;
@@ -154,7 +167,6 @@ static PHP_METHOD(DBPointer, unserialize)
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -228,14 +240,14 @@ zend_object* php_phongo_dbpointer_create_object(zend_class_entry* class_type) /*
 	return &intern->std;
 } /* }}} */
 
-static zend_object* php_phongo_dbpointer_clone_object(zval* object) /* {{{ */
+static zend_object* php_phongo_dbpointer_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	php_phongo_dbpointer_t* intern;
 	php_phongo_dbpointer_t* new_intern;
 	zend_object*            new_object;
 
-	intern     = Z_DBPOINTER_OBJ_P(object);
-	new_object = php_phongo_dbpointer_create_object(Z_OBJCE_P(object));
+	intern     = Z_OBJ_DBPOINTER(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_dbpointer_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
 	new_intern = Z_OBJ_DBPOINTER(new_object);
 	zend_objects_clone_members(&new_intern->std, &intern->std);
@@ -250,6 +262,8 @@ static int php_phongo_dbpointer_compare_objects(zval* o1, zval* o2) /* {{{ */
 	php_phongo_dbpointer_t *intern1, *intern2;
 	int                     retval;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_DBPOINTER_OBJ_P(o1);
 	intern2 = Z_DBPOINTER_OBJ_P(o2);
 
@@ -262,12 +276,12 @@ static int php_phongo_dbpointer_compare_objects(zval* o1, zval* o2) /* {{{ */
 	return strcmp(intern1->id, intern2->id);
 } /* }}} */
 
-HashTable* php_phongo_dbpointer_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+HashTable* php_phongo_dbpointer_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_dbpointer_t* intern;
 	HashTable*              props;
 
-	intern = Z_DBPOINTER_OBJ_P(object);
+	intern = Z_OBJ_DBPOINTER(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
 
@@ -287,13 +301,13 @@ HashTable* php_phongo_dbpointer_get_properties_hash(zval* object, bool is_debug)
 	return props;
 } /* }}} */
 
-static HashTable* php_phongo_dbpointer_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_dbpointer_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_dbpointer_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_dbpointer_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_dbpointer_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_dbpointer_get_properties_hash(object, false);
 } /* }}} */
@@ -313,12 +327,12 @@ void php_phongo_dbpointer_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_dbpointer_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_dbpointer, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_dbpointer.clone_obj       = php_phongo_dbpointer_clone_object;
-	php_phongo_handler_dbpointer.compare_objects = php_phongo_dbpointer_compare_objects;
-	php_phongo_handler_dbpointer.get_debug_info  = php_phongo_dbpointer_get_debug_info;
-	php_phongo_handler_dbpointer.get_properties  = php_phongo_dbpointer_get_properties;
-	php_phongo_handler_dbpointer.free_obj        = php_phongo_dbpointer_free_object;
-	php_phongo_handler_dbpointer.offset          = XtOffsetOf(php_phongo_dbpointer_t, std);
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(dbpointer);
+	php_phongo_handler_dbpointer.clone_obj      = php_phongo_dbpointer_clone_object;
+	php_phongo_handler_dbpointer.get_debug_info = php_phongo_dbpointer_get_debug_info;
+	php_phongo_handler_dbpointer.get_properties = php_phongo_dbpointer_get_properties;
+	php_phongo_handler_dbpointer.free_obj       = php_phongo_dbpointer_free_object;
+	php_phongo_handler_dbpointer.offset         = XtOffsetOf(php_phongo_dbpointer_t, std);
 } /* }}} */
 
 /*
