@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2017 MongoDB, Inc.
+ * Copyright 2015-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -60,14 +60,14 @@ static bool php_phongo_decimal128_init_from_hash(php_phongo_decimal128_t* intern
    Construct a new BSON Decimal128 type */
 static PHP_METHOD(Decimal128, __construct)
 {
-	php_phongo_decimal128_t* intern;
 	zend_error_handling      error_handling;
+	php_phongo_decimal128_t* intern;
 	char*                    value;
 	size_t                   value_len;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &value, &value_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -77,17 +77,21 @@ static PHP_METHOD(Decimal128, __construct)
 	php_phongo_decimal128_init(intern, value);
 } /* }}} */
 
-/* {{{ proto void MongoDB\BSON\Decimal128::__set_state(array $properties)
+/* {{{ proto MongoDB\BSON\Decimal128 MongoDB\BSON\Decimal128::__set_state(array $properties)
 */
 static PHP_METHOD(Decimal128, __set_state)
 {
+	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	HashTable*               props;
 	zval*                    array;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_decimal128_ce);
 
@@ -101,14 +105,18 @@ static PHP_METHOD(Decimal128, __set_state)
 */
 static PHP_METHOD(Decimal128, __toString)
 {
+	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	char                     outbuf[BSON_DECIMAL128_STRING];
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	bson_decimal128_to_string(&intern->decimal, outbuf);
 
@@ -119,12 +127,16 @@ static PHP_METHOD(Decimal128, __toString)
 */
 static PHP_METHOD(Decimal128, jsonSerialize)
 {
+	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	char                     outbuf[BSON_DECIMAL128_STRING] = "";
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
@@ -137,6 +149,7 @@ static PHP_METHOD(Decimal128, jsonSerialize)
 */
 static PHP_METHOD(Decimal128, serialize)
 {
+	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	zval                     retval;
 	php_serialize_data_t     var_hash;
@@ -145,9 +158,12 @@ static PHP_METHOD(Decimal128, serialize)
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	bson_decimal128_to_string(&intern->decimal, outbuf);
 	array_init_size(&retval, 1);
@@ -168,8 +184,8 @@ static PHP_METHOD(Decimal128, serialize)
 */
 static PHP_METHOD(Decimal128, unserialize)
 {
-	php_phongo_decimal128_t* intern;
 	zend_error_handling      error_handling;
+	php_phongo_decimal128_t* intern;
 	char*                    serialized;
 	size_t                   serialized_len;
 	zval                     props;
@@ -178,7 +194,6 @@ static PHP_METHOD(Decimal128, unserialize)
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -257,14 +272,14 @@ static zend_object* php_phongo_decimal128_create_object(zend_class_entry* class_
 	return &intern->std;
 } /* }}} */
 
-static zend_object* php_phongo_decimal128_clone_object(zval* object) /* {{{ */
+static zend_object* php_phongo_decimal128_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	php_phongo_decimal128_t* intern;
 	php_phongo_decimal128_t* new_intern;
 	zend_object*             new_object;
 
-	intern     = Z_DECIMAL128_OBJ_P(object);
-	new_object = php_phongo_decimal128_create_object(Z_OBJCE_P(object));
+	intern     = Z_OBJ_DECIMAL128(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_decimal128_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
 	new_intern = Z_OBJ_DECIMAL128(new_object);
 	zend_objects_clone_members(&new_intern->std, &intern->std);
@@ -276,13 +291,13 @@ static zend_object* php_phongo_decimal128_clone_object(zval* object) /* {{{ */
 	return new_object;
 } /* }}} */
 
-static HashTable* php_phongo_decimal128_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+static HashTable* php_phongo_decimal128_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_decimal128_t* intern;
 	HashTable*               props;
 	char                     outbuf[BSON_DECIMAL128_STRING] = "";
 
-	intern = Z_DECIMAL128_OBJ_P(object);
+	intern = Z_OBJ_DECIMAL128(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 1);
 
@@ -302,13 +317,13 @@ static HashTable* php_phongo_decimal128_get_properties_hash(zval* object, bool i
 	return props;
 } /* }}} */
 
-static HashTable* php_phongo_decimal128_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_decimal128_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_decimal128_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_decimal128_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_decimal128_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_decimal128_get_properties_hash(object, false);
 } /* }}} */
