@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -78,15 +78,15 @@ static bool php_phongo_javascript_init_from_hash(php_phongo_javascript_t* intern
    be evaluated. Note that this type cannot be represented as Extended JSON. */
 static PHP_METHOD(Javascript, __construct)
 {
-	php_phongo_javascript_t* intern;
 	zend_error_handling      error_handling;
+	php_phongo_javascript_t* intern;
 	char*                    code;
 	size_t                   code_len;
 	zval*                    scope = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|A!", &code, &code_len, &scope) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -96,17 +96,21 @@ static PHP_METHOD(Javascript, __construct)
 	php_phongo_javascript_init(intern, code, code_len, scope);
 } /* }}} */
 
-/* {{{ proto void MongoDB\BSON\Javascript::__set_state(array $properties)
+/* {{{ proto MongoDB\BSON\Javascript MongoDB\BSON\Javascript::__set_state(array $properties)
 */
 static PHP_METHOD(Javascript, __set_state)
 {
+	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 	HashTable*               props;
 	zval*                    array;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_javascript_ce);
 
@@ -120,11 +124,15 @@ static PHP_METHOD(Javascript, __set_state)
    Return the Javascript's code string. */
 static PHP_METHOD(Javascript, __toString)
 {
+	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
@@ -135,11 +143,15 @@ static PHP_METHOD(Javascript, __toString)
 */
 static PHP_METHOD(Javascript, getCode)
 {
+	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
@@ -150,11 +162,15 @@ static PHP_METHOD(Javascript, getCode)
 */
 static PHP_METHOD(Javascript, getScope)
 {
+	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
@@ -182,11 +198,15 @@ static PHP_METHOD(Javascript, getScope)
 */
 static PHP_METHOD(Javascript, jsonSerialize)
 {
+	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
@@ -210,6 +230,7 @@ static PHP_METHOD(Javascript, jsonSerialize)
 */
 static PHP_METHOD(Javascript, serialize)
 {
+	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 	zval                     retval;
 	php_phongo_bson_state    state;
@@ -220,9 +241,12 @@ static PHP_METHOD(Javascript, serialize)
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (intern->scope && intern->scope->len) {
 		if (!php_phongo_bson_to_zval_ex(bson_get_data(intern->scope), intern->scope->len, &state)) {
@@ -252,8 +276,8 @@ static PHP_METHOD(Javascript, serialize)
 */
 static PHP_METHOD(Javascript, unserialize)
 {
-	php_phongo_javascript_t* intern;
 	zend_error_handling      error_handling;
+	php_phongo_javascript_t* intern;
 	char*                    serialized;
 	size_t                   serialized_len;
 	zval                     props;
@@ -262,7 +286,6 @@ static PHP_METHOD(Javascript, unserialize)
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -351,14 +374,14 @@ zend_object* php_phongo_javascript_create_object(zend_class_entry* class_type) /
 	return &intern->std;
 } /* }}} */
 
-static zend_object* php_phongo_javascript_clone_object(zval* object) /* {{{ */
+static zend_object* php_phongo_javascript_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	php_phongo_javascript_t* intern;
 	php_phongo_javascript_t* new_intern;
 	zend_object*             new_object;
 
-	intern     = Z_JAVASCRIPT_OBJ_P(object);
-	new_object = php_phongo_javascript_create_object(Z_OBJCE_P(object));
+	intern     = Z_OBJ_JAVASCRIPT(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_javascript_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
 	new_intern = Z_OBJ_JAVASCRIPT(new_object);
 	zend_objects_clone_members(&new_intern->std, &intern->std);
@@ -373,6 +396,8 @@ static int php_phongo_javascript_compare_objects(zval* o1, zval* o2) /* {{{ */
 {
 	php_phongo_javascript_t *intern1, *intern2;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_JAVASCRIPT_OBJ_P(o1);
 	intern2 = Z_JAVASCRIPT_OBJ_P(o2);
 
@@ -380,12 +405,12 @@ static int php_phongo_javascript_compare_objects(zval* o1, zval* o2) /* {{{ */
 	return strcmp(intern1->code, intern2->code);
 } /* }}} */
 
-HashTable* php_phongo_javascript_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+HashTable* php_phongo_javascript_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_javascript_t* intern;
 	HashTable*               props;
 
-	intern = Z_JAVASCRIPT_OBJ_P(object);
+	intern = Z_OBJ_JAVASCRIPT(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
 
@@ -424,13 +449,13 @@ failure:
 	return NULL;
 } /* }}} */
 
-static HashTable* php_phongo_javascript_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_javascript_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_javascript_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_javascript_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_javascript_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_javascript_get_properties_hash(object, false);
 } /* }}} */
@@ -451,12 +476,12 @@ void php_phongo_javascript_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_javascript_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_javascript, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_javascript.clone_obj       = php_phongo_javascript_clone_object;
-	php_phongo_handler_javascript.compare_objects = php_phongo_javascript_compare_objects;
-	php_phongo_handler_javascript.get_debug_info  = php_phongo_javascript_get_debug_info;
-	php_phongo_handler_javascript.get_properties  = php_phongo_javascript_get_properties;
-	php_phongo_handler_javascript.free_obj        = php_phongo_javascript_free_object;
-	php_phongo_handler_javascript.offset          = XtOffsetOf(php_phongo_javascript_t, std);
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(javascript);
+	php_phongo_handler_javascript.clone_obj      = php_phongo_javascript_clone_object;
+	php_phongo_handler_javascript.get_debug_info = php_phongo_javascript_get_debug_info;
+	php_phongo_handler_javascript.get_properties = php_phongo_javascript_get_properties;
+	php_phongo_handler_javascript.free_obj       = php_phongo_javascript_free_object;
+	php_phongo_handler_javascript.offset         = XtOffsetOf(php_phongo_javascript_t, std);
 } /* }}} */
 
 /*
