@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,15 +28,19 @@
 
 zend_class_entry* php_phongo_minkey_ce;
 
-/* {{{ proto void MongoDB\BSON\MinKey::__set_state(array $properties)
+/* {{{ proto MongoDB\BSON\MinKey MongoDB\BSON\MinKey::__set_state(array $properties)
 */
 static PHP_METHOD(MinKey, __set_state)
 {
-	zval* array;
+	zend_error_handling error_handling;
+	zval*               array;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_minkey_ce);
 } /* }}} */
@@ -45,9 +49,14 @@ static PHP_METHOD(MinKey, __set_state)
 */
 static PHP_METHOD(MinKey, jsonSerialize)
 {
+	zend_error_handling error_handling;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	array_init_size(return_value, 1);
 	ADD_ASSOC_LONG_EX(return_value, "$minKey", 1);
@@ -57,6 +66,15 @@ static PHP_METHOD(MinKey, jsonSerialize)
 */
 static PHP_METHOD(MinKey, serialize)
 {
+	zend_error_handling error_handling;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
+	}
+	zend_restore_error_handling(&error_handling);
+
 	RETURN_STRING("");
 } /* }}} */
 
@@ -69,7 +87,6 @@ static PHP_METHOD(MinKey, unserialize)
 	size_t              serialized_len;
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
