@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -86,16 +86,16 @@ static bool php_phongo_regex_init_from_hash(php_phongo_regex_t* intern, HashTabl
    Constructs a new BSON regular expression type. */
 static PHP_METHOD(Regex, __construct)
 {
-	php_phongo_regex_t* intern;
 	zend_error_handling error_handling;
+	php_phongo_regex_t* intern;
 	char*               pattern;
 	size_t              pattern_len;
 	char*               flags     = NULL;
 	size_t              flags_len = 0;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_REGEX_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|s", &pattern, &pattern_len, &flags, &flags_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -109,13 +109,17 @@ static PHP_METHOD(Regex, __construct)
 */
 static PHP_METHOD(Regex, getPattern)
 {
+	zend_error_handling error_handling;
 	php_phongo_regex_t* intern;
 
 	intern = Z_REGEX_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_STRINGL(intern->pattern, intern->pattern_len);
 } /* }}} */
@@ -124,28 +128,36 @@ static PHP_METHOD(Regex, getPattern)
 */
 static PHP_METHOD(Regex, getFlags)
 {
+	zend_error_handling error_handling;
 	php_phongo_regex_t* intern;
 
 	intern = Z_REGEX_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_STRINGL(intern->flags, intern->flags_len);
 } /* }}} */
 
-/* {{{ proto void MongoDB\BSON\Regex::__set_state(array $properties)
+/* {{{ proto MongoDB\BSON\Regex MongoDB\BSON\Regex::__set_state(array $properties)
 */
 static PHP_METHOD(Regex, __set_state)
 {
+	zend_error_handling error_handling;
 	php_phongo_regex_t* intern;
 	HashTable*          props;
 	zval*               array;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_regex_ce);
 
@@ -159,15 +171,19 @@ static PHP_METHOD(Regex, __set_state)
    Returns a string in the form: /pattern/flags */
 static PHP_METHOD(Regex, __toString)
 {
+	zend_error_handling error_handling;
 	php_phongo_regex_t* intern;
 	char*               regex;
 	int                 regex_len;
 
 	intern = Z_REGEX_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	regex_len = spprintf(&regex, 0, "/%s/%s", intern->pattern, intern->flags);
 	RETVAL_STRINGL(regex, regex_len);
@@ -178,11 +194,15 @@ static PHP_METHOD(Regex, __toString)
 */
 static PHP_METHOD(Regex, jsonSerialize)
 {
+	zend_error_handling error_handling;
 	php_phongo_regex_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_REGEX_OBJ_P(getThis());
 
@@ -195,6 +215,7 @@ static PHP_METHOD(Regex, jsonSerialize)
 */
 static PHP_METHOD(Regex, serialize)
 {
+	zend_error_handling  error_handling;
 	php_phongo_regex_t*  intern;
 	zval                 retval;
 	php_serialize_data_t var_hash;
@@ -202,9 +223,12 @@ static PHP_METHOD(Regex, serialize)
 
 	intern = Z_REGEX_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	array_init_size(&retval, 2);
 	ADD_ASSOC_STRINGL(&retval, "pattern", intern->pattern, intern->pattern_len);
@@ -225,8 +249,8 @@ static PHP_METHOD(Regex, serialize)
 */
 static PHP_METHOD(Regex, unserialize)
 {
-	php_phongo_regex_t*    intern;
 	zend_error_handling    error_handling;
+	php_phongo_regex_t*    intern;
 	char*                  serialized;
 	size_t                 serialized_len;
 	zval                   props;
@@ -235,7 +259,6 @@ static PHP_METHOD(Regex, unserialize)
 	intern = Z_REGEX_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -257,7 +280,7 @@ static PHP_METHOD(Regex, unserialize)
 } /* }}} */
 
 /* {{{ MongoDB\BSON\Regex function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_Regex___construct, 0, 0, 2)
+ZEND_BEGIN_ARG_INFO_EX(ai_Regex___construct, 0, 0, 1)
 	ZEND_ARG_INFO(0, pattern)
 	ZEND_ARG_INFO(0, flags)
 ZEND_END_ARG_INFO()
@@ -325,14 +348,14 @@ static zend_object* php_phongo_regex_create_object(zend_class_entry* class_type)
 	return &intern->std;
 } /* }}} */
 
-static zend_object* php_phongo_regex_clone_object(zval* object) /* {{{ */
+static zend_object* php_phongo_regex_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	php_phongo_regex_t* intern;
 	php_phongo_regex_t* new_intern;
 	zend_object*        new_object;
 
-	intern     = Z_REGEX_OBJ_P(object);
-	new_object = php_phongo_regex_create_object(Z_OBJCE_P(object));
+	intern     = Z_OBJ_REGEX(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_regex_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
 	new_intern = Z_OBJ_REGEX(new_object);
 	zend_objects_clone_members(&new_intern->std, &intern->std);
@@ -347,6 +370,8 @@ static int php_phongo_regex_compare_objects(zval* o1, zval* o2) /* {{{ */
 	php_phongo_regex_t *intern1, *intern2;
 	int                 retval;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_REGEX_OBJ_P(o1);
 	intern2 = Z_REGEX_OBJ_P(o2);
 
@@ -360,12 +385,12 @@ static int php_phongo_regex_compare_objects(zval* o1, zval* o2) /* {{{ */
 	return strcmp(intern1->flags, intern2->flags);
 } /* }}} */
 
-static HashTable* php_phongo_regex_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+static HashTable* php_phongo_regex_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_regex_t* intern;
 	HashTable*          props;
 
-	intern = Z_REGEX_OBJ_P(object);
+	intern = Z_OBJ_REGEX(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
 
@@ -386,13 +411,13 @@ static HashTable* php_phongo_regex_get_properties_hash(zval* object, bool is_deb
 	return props;
 } /* }}} */
 
-static HashTable* php_phongo_regex_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_regex_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_regex_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_regex_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_regex_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_regex_get_properties_hash(object, false);
 } /* }}} */
@@ -413,12 +438,12 @@ void php_phongo_regex_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_regex_ce, 1, php_phongo_json_serializable_ce);
 
 	memcpy(&php_phongo_handler_regex, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_regex.clone_obj       = php_phongo_regex_clone_object;
-	php_phongo_handler_regex.compare_objects = php_phongo_regex_compare_objects;
-	php_phongo_handler_regex.get_debug_info  = php_phongo_regex_get_debug_info;
-	php_phongo_handler_regex.get_properties  = php_phongo_regex_get_properties;
-	php_phongo_handler_regex.free_obj        = php_phongo_regex_free_object;
-	php_phongo_handler_regex.offset          = XtOffsetOf(php_phongo_regex_t, std);
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(regex);
+	php_phongo_handler_regex.clone_obj      = php_phongo_regex_clone_object;
+	php_phongo_handler_regex.get_debug_info = php_phongo_regex_get_debug_info;
+	php_phongo_handler_regex.get_properties = php_phongo_regex_get_properties;
+	php_phongo_handler_regex.free_obj       = php_phongo_regex_free_object;
+	php_phongo_handler_regex.offset         = XtOffsetOf(php_phongo_regex_t, std);
 } /* }}} */
 
 /*
