@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -62,11 +62,15 @@ static bool php_phongo_symbol_init_from_hash(php_phongo_symbol_t* intern, HashTa
    Return the Symbol's symbol string. */
 static PHP_METHOD(Symbol, __toString)
 {
+	zend_error_handling  error_handling;
 	php_phongo_symbol_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_SYMBOL_OBJ_P(getThis());
 
@@ -77,11 +81,15 @@ static PHP_METHOD(Symbol, __toString)
 */
 static PHP_METHOD(Symbol, jsonSerialize)
 {
+	zend_error_handling  error_handling;
 	php_phongo_symbol_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_SYMBOL_OBJ_P(getThis());
 
@@ -93,6 +101,7 @@ static PHP_METHOD(Symbol, jsonSerialize)
 */
 static PHP_METHOD(Symbol, serialize)
 {
+	zend_error_handling  error_handling;
 	php_phongo_symbol_t* intern;
 	zval                 retval;
 	php_serialize_data_t var_hash;
@@ -100,9 +109,12 @@ static PHP_METHOD(Symbol, serialize)
 
 	intern = Z_SYMBOL_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	array_init_size(&retval, 1);
 	ADD_ASSOC_STRINGL(&retval, "symbol", intern->symbol, intern->symbol_len);
@@ -122,8 +134,8 @@ static PHP_METHOD(Symbol, serialize)
 */
 static PHP_METHOD(Symbol, unserialize)
 {
-	php_phongo_symbol_t*   intern;
 	zend_error_handling    error_handling;
+	php_phongo_symbol_t*   intern;
 	char*                  serialized;
 	size_t                 serialized_len;
 	zval                   props;
@@ -132,7 +144,6 @@ static PHP_METHOD(Symbol, unserialize)
 	intern = Z_SYMBOL_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -206,14 +217,14 @@ zend_object* php_phongo_symbol_create_object(zend_class_entry* class_type) /* {{
 	return &intern->std;
 } /* }}} */
 
-static zend_object* php_phongo_symbol_clone_object(zval* object) /* {{{ */
+static zend_object* php_phongo_symbol_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	php_phongo_symbol_t* intern;
 	php_phongo_symbol_t* new_intern;
 	zend_object*         new_object;
 
-	intern     = Z_SYMBOL_OBJ_P(object);
-	new_object = php_phongo_symbol_create_object(Z_OBJCE_P(object));
+	intern     = Z_OBJ_SYMBOL(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_symbol_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
 	new_intern = Z_OBJ_SYMBOL(new_object);
 	zend_objects_clone_members(&new_intern->std, &intern->std);
@@ -227,18 +238,20 @@ static int php_phongo_symbol_compare_objects(zval* o1, zval* o2) /* {{{ */
 {
 	php_phongo_symbol_t *intern1, *intern2;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_SYMBOL_OBJ_P(o1);
 	intern2 = Z_SYMBOL_OBJ_P(o2);
 
 	return strcmp(intern1->symbol, intern2->symbol);
 } /* }}} */
 
-HashTable* php_phongo_symbol_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+HashTable* php_phongo_symbol_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_symbol_t* intern;
 	HashTable*           props;
 
-	intern = Z_SYMBOL_OBJ_P(object);
+	intern = Z_OBJ_SYMBOL(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
 
@@ -256,13 +269,13 @@ HashTable* php_phongo_symbol_get_properties_hash(zval* object, bool is_debug) /*
 	return props;
 } /* }}} */
 
-static HashTable* php_phongo_symbol_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_symbol_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_symbol_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_symbol_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_symbol_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_symbol_get_properties_hash(object, false);
 } /* }}} */
@@ -282,12 +295,12 @@ void php_phongo_symbol_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_symbol_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_symbol, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_symbol.clone_obj       = php_phongo_symbol_clone_object;
-	php_phongo_handler_symbol.compare_objects = php_phongo_symbol_compare_objects;
-	php_phongo_handler_symbol.get_debug_info  = php_phongo_symbol_get_debug_info;
-	php_phongo_handler_symbol.get_properties  = php_phongo_symbol_get_properties;
-	php_phongo_handler_symbol.free_obj        = php_phongo_symbol_free_object;
-	php_phongo_handler_symbol.offset          = XtOffsetOf(php_phongo_symbol_t, std);
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(symbol);
+	php_phongo_handler_symbol.clone_obj      = php_phongo_symbol_clone_object;
+	php_phongo_handler_symbol.get_debug_info = php_phongo_symbol_get_debug_info;
+	php_phongo_handler_symbol.get_properties = php_phongo_symbol_get_properties;
+	php_phongo_handler_symbol.free_obj       = php_phongo_symbol_free_object;
+	php_phongo_handler_symbol.offset         = XtOffsetOf(php_phongo_symbol_t, std);
 } /* }}} */
 
 /*
