@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -94,13 +94,13 @@ static bool php_phongo_timestamp_init_from_hash(php_phongo_timestamp_t* intern,
    4-byte timestamp. */
 static PHP_METHOD(Timestamp, __construct)
 {
-	php_phongo_timestamp_t* intern;
 	zend_error_handling     error_handling;
+	php_phongo_timestamp_t* intern;
 	zval *                  increment = NULL, *timestamp = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "zz", &increment, &timestamp) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -137,13 +137,17 @@ static PHP_METHOD(Timestamp, __construct)
 */
 static PHP_METHOD(Timestamp, getIncrement)
 {
+	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETVAL_LONG(intern->increment);
 } /* }}} */
@@ -152,28 +156,36 @@ static PHP_METHOD(Timestamp, getIncrement)
 */
 static PHP_METHOD(Timestamp, getTimestamp)
 {
+	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETVAL_LONG(intern->timestamp);
 } /* }}} */
 
-/* {{{ proto void MongoDB\BSON\Timestamp::__set_state(array $properties)
+/* {{{ proto MongoDB\BSON\Timestamp MongoDB\BSON\Timestamp::__set_state(array $properties)
 */
 static PHP_METHOD(Timestamp, __set_state)
 {
+	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	HashTable*              props;
 	zval*                   array;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_timestamp_ce);
 
@@ -187,15 +199,19 @@ static PHP_METHOD(Timestamp, __set_state)
    Returns a string in the form: [increment:timestamp] */
 static PHP_METHOD(Timestamp, __toString)
 {
+	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	char*                   retval;
 	int                     retval_len;
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	retval_len = spprintf(&retval, 0, "[%" PRIu32 ":%" PRIu32 "]", intern->increment, intern->timestamp);
 	RETVAL_STRINGL(retval, retval_len);
@@ -206,11 +222,15 @@ static PHP_METHOD(Timestamp, __toString)
 */
 static PHP_METHOD(Timestamp, jsonSerialize)
 {
+	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
@@ -230,6 +250,7 @@ static PHP_METHOD(Timestamp, jsonSerialize)
 */
 static PHP_METHOD(Timestamp, serialize)
 {
+	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	zval                    retval;
 	php_serialize_data_t    var_hash;
@@ -241,9 +262,12 @@ static PHP_METHOD(Timestamp, serialize)
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	s_increment_len = snprintf(s_increment, sizeof(s_increment), "%" PRIu32, intern->increment);
 	s_timestamp_len = snprintf(s_timestamp, sizeof(s_timestamp), "%" PRIu32, intern->timestamp);
@@ -267,8 +291,8 @@ static PHP_METHOD(Timestamp, serialize)
 */
 static PHP_METHOD(Timestamp, unserialize)
 {
-	php_phongo_timestamp_t* intern;
 	zend_error_handling     error_handling;
+	php_phongo_timestamp_t* intern;
 	char*                   serialized;
 	size_t                  serialized_len;
 	zval                    props;
@@ -277,7 +301,6 @@ static PHP_METHOD(Timestamp, unserialize)
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -359,14 +382,14 @@ static zend_object* php_phongo_timestamp_create_object(zend_class_entry* class_t
 	return &intern->std;
 } /* }}} */
 
-static zend_object* php_phongo_timestamp_clone_object(zval* object) /* {{{ */
+static zend_object* php_phongo_timestamp_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	php_phongo_timestamp_t* intern;
 	php_phongo_timestamp_t* new_intern;
 	zend_object*            new_object;
 
-	intern     = Z_TIMESTAMP_OBJ_P(object);
-	new_object = php_phongo_timestamp_create_object(Z_OBJCE_P(object));
+	intern     = Z_OBJ_TIMESTAMP(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_timestamp_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
 	new_intern = Z_OBJ_TIMESTAMP(new_object);
 	zend_objects_clone_members(&new_intern->std, &intern->std);
@@ -380,6 +403,8 @@ static int php_phongo_timestamp_compare_objects(zval* o1, zval* o2) /* {{{ */
 {
 	php_phongo_timestamp_t *intern1, *intern2;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_TIMESTAMP_OBJ_P(o1);
 	intern2 = Z_TIMESTAMP_OBJ_P(o2);
 
@@ -395,7 +420,7 @@ static int php_phongo_timestamp_compare_objects(zval* o1, zval* o2) /* {{{ */
 	return 0;
 } /* }}} */
 
-static HashTable* php_phongo_timestamp_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+static HashTable* php_phongo_timestamp_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_timestamp_t* intern;
 	HashTable*              props;
@@ -404,7 +429,7 @@ static HashTable* php_phongo_timestamp_get_properties_hash(zval* object, bool is
 	int                     s_increment_len;
 	int                     s_timestamp_len;
 
-	intern = Z_TIMESTAMP_OBJ_P(object);
+	intern = Z_OBJ_TIMESTAMP(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
 
@@ -428,13 +453,13 @@ static HashTable* php_phongo_timestamp_get_properties_hash(zval* object, bool is
 	return props;
 } /* }}} */
 
-static HashTable* php_phongo_timestamp_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_timestamp_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_timestamp_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_timestamp_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_timestamp_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_timestamp_get_properties_hash(object, false);
 } /* }}} */
@@ -455,12 +480,12 @@ void php_phongo_timestamp_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_timestamp_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_timestamp, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_timestamp.clone_obj       = php_phongo_timestamp_clone_object;
-	php_phongo_handler_timestamp.compare_objects = php_phongo_timestamp_compare_objects;
-	php_phongo_handler_timestamp.get_debug_info  = php_phongo_timestamp_get_debug_info;
-	php_phongo_handler_timestamp.get_properties  = php_phongo_timestamp_get_properties;
-	php_phongo_handler_timestamp.free_obj        = php_phongo_timestamp_free_object;
-	php_phongo_handler_timestamp.offset          = XtOffsetOf(php_phongo_timestamp_t, std);
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(timestamp);
+	php_phongo_handler_timestamp.clone_obj      = php_phongo_timestamp_clone_object;
+	php_phongo_handler_timestamp.get_debug_info = php_phongo_timestamp_get_debug_info;
+	php_phongo_handler_timestamp.get_properties = php_phongo_timestamp_get_properties;
+	php_phongo_handler_timestamp.free_obj       = php_phongo_timestamp_free_object;
+	php_phongo_handler_timestamp.offset         = XtOffsetOf(php_phongo_timestamp_t, std);
 } /* }}} */
 
 /*
