@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2017 MongoDB, Inc.
+ * Copyright 2015-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,6 @@
 
 #include <php.h>
 #include <Zend/zend_interfaces.h>
-#include <ext/spl/spl_iterators.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,6 +26,11 @@
 #include "php_phongo.h"
 #include "php_bson.h"
 
+#if PHP_VERSION_ID < 70200
+#include <ext/spl/spl_iterators.h>
+#define zend_ce_countable spl_ce_Countable
+#endif /* PHP_VERSION_ID < 70200 */
+
 #define PHONGO_BULKWRITE_BYPASS_UNSET -1
 
 zend_class_entry* php_phongo_bulkwrite_ce;
@@ -312,14 +316,14 @@ static bool php_phongo_bulkwrite_delete_apply_options(bson_t* boptions, zval* zo
    Constructs a new BulkWrite */
 static PHP_METHOD(BulkWrite, __construct)
 {
-	php_phongo_bulkwrite_t* intern;
 	zend_error_handling     error_handling;
+	php_phongo_bulkwrite_t* intern;
 	zval*                   options = NULL;
 	zend_bool               ordered = 1;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -347,6 +351,7 @@ static PHP_METHOD(BulkWrite, __construct)
    Adds an insert operation to the BulkWrite */
 static PHP_METHOD(BulkWrite, insert)
 {
+	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 	zval*                   zdocument;
 	bson_t                  bdocument = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
@@ -356,9 +361,12 @@ static PHP_METHOD(BulkWrite, insert)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A", &zdocument) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	bson_flags |= PHONGO_BSON_RETURN_ID;
 
@@ -392,6 +400,7 @@ cleanup:
    Adds an update operation to the BulkWrite */
 static PHP_METHOD(BulkWrite, update)
 {
+	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 	zval *                  zquery, *zupdate, *zoptions = NULL;
 	bson_t                  bquery = BSON_INITIALIZER, bupdate = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
@@ -399,9 +408,12 @@ static PHP_METHOD(BulkWrite, update)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "AA|a!", &zquery, &zupdate, &zoptions) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	php_phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, &bquery, NULL);
 
@@ -455,6 +467,7 @@ cleanup:
    Adds a delete operation to the BulkWrite */
 static PHP_METHOD(BulkWrite, delete)
 {
+	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 	zval *                  zquery, *zoptions = NULL;
 	bson_t                  bquery = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
@@ -462,9 +475,12 @@ static PHP_METHOD(BulkWrite, delete)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A|a!", &zquery, &zoptions) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	php_phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, &bquery, NULL);
 
@@ -499,13 +515,17 @@ cleanup:
    Returns the number of operations that have been added to the BulkWrite */
 static PHP_METHOD(BulkWrite, count)
 {
+	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_LONG(intern->num_ops);
 } /* }}} */
@@ -566,6 +586,10 @@ static void php_phongo_bulkwrite_free_object(zend_object* object) /* {{{ */
 	if (intern->collection) {
 		efree(intern->collection);
 	}
+
+	if (!Z_ISUNDEF(intern->session)) {
+		zval_ptr_dtor(&intern->session);
+	}
 } /* }}} */
 
 static zend_object* php_phongo_bulkwrite_create_object(zend_class_entry* class_type) /* {{{ */
@@ -582,13 +606,13 @@ static zend_object* php_phongo_bulkwrite_create_object(zend_class_entry* class_t
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_bulkwrite_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	zval                    retval = ZVAL_STATIC_INIT;
 	php_phongo_bulkwrite_t* intern = NULL;
 
 	*is_temp = 1;
-	intern   = Z_BULKWRITE_OBJ_P(object);
+	intern   = Z_OBJ_BULKWRITE(PHONGO_COMPAT_GET_OBJ(object));
 	array_init(&retval);
 
 	if (intern->database) {
@@ -614,6 +638,13 @@ static HashTable* php_phongo_bulkwrite_get_debug_info(zval* object, int* is_temp
 	ADD_ASSOC_BOOL_EX(&retval, "executed", intern->executed);
 	ADD_ASSOC_LONG_EX(&retval, "server_id", mongoc_bulk_operation_get_hint(intern->bulk));
 
+	if (!Z_ISUNDEF(intern->session)) {
+		ADD_ASSOC_ZVAL_EX(&retval, "session", &intern->session);
+		Z_ADDREF(intern->session);
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "session");
+	}
+
 	if (mongoc_bulk_operation_get_write_concern(intern->bulk)) {
 		zval write_concern;
 
@@ -642,7 +673,7 @@ void php_phongo_bulkwrite_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_bulkwrite.free_obj       = php_phongo_bulkwrite_free_object;
 	php_phongo_handler_bulkwrite.offset         = XtOffsetOf(php_phongo_bulkwrite_t, std);
 
-	zend_class_implements(php_phongo_bulkwrite_ce, 1, spl_ce_Countable);
+	zend_class_implements(php_phongo_bulkwrite_ce, 1, zend_ce_countable);
 } /* }}} */
 
 /*
