@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,158 +52,11 @@ static void php_phongo_cursor_free_current(php_phongo_cursor_t* cursor) /* {{{ *
 	}
 } /* }}} */
 
-/* {{{ MongoDB\Driver\Cursor iterator handlers */
-static void php_phongo_cursor_iterator_dtor(zend_object_iterator* iter) /* {{{ */
-{
-	php_phongo_cursor_iterator* cursor_it = (php_phongo_cursor_iterator*) iter;
-
-	if (!Z_ISUNDEF(cursor_it->intern.data)) {
-		zval_ptr_dtor(&cursor_it->intern.data);
-	}
-} /* }}} */
-
-static int php_phongo_cursor_iterator_valid(zend_object_iterator* iter) /* {{{ */
-{
-	php_phongo_cursor_t* cursor = ((php_phongo_cursor_iterator*) iter)->cursor;
-
-	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
-		return SUCCESS;
-	}
-
-	return FAILURE;
-} /* }}} */
-
-static void php_phongo_cursor_iterator_get_current_key(zend_object_iterator* iter, zval* key) /* {{{ */
-{
-	php_phongo_cursor_t* cursor = ((php_phongo_cursor_iterator*) iter)->cursor;
-
-	ZVAL_LONG(key, cursor->current);
-} /* }}} */
-
-static zval* php_phongo_cursor_iterator_get_current_data(zend_object_iterator* iter) /* {{{ */
-{
-	php_phongo_cursor_t* cursor = ((php_phongo_cursor_iterator*) iter)->cursor;
-
-	return &cursor->visitor_data.zchild;
-} /* }}} */
-
-static void php_phongo_cursor_iterator_move_forward(zend_object_iterator* iter) /* {{{ */
-{
-	php_phongo_cursor_iterator* cursor_it = (php_phongo_cursor_iterator*) iter;
-	php_phongo_cursor_t*        cursor    = cursor_it->cursor;
-	const bson_t*               doc;
-
-	php_phongo_cursor_free_current(cursor);
-
-	/* If the cursor has already advanced, increment its position. Otherwise,
-	 * the first call to mongoc_cursor_next() will be made below and we should
-	 * leave its position at zero. */
-	if (cursor->advanced) {
-		cursor->current++;
-	} else {
-		cursor->advanced = true;
-	}
-
-	if (mongoc_cursor_next(cursor->cursor, &doc)) {
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data)) {
-			/* Free invalid result, but don't return as we want to free the
-			 * session if the cursor is exhausted. */
-			php_phongo_cursor_free_current(cursor);
-		}
-	} else {
-		bson_error_t  error = { 0 };
-		const bson_t* doc   = NULL;
-
-		if (mongoc_cursor_error_document(cursor->cursor, &error, &doc)) {
-			/* Intentionally not destroying the cursor as it will happen
-			 * naturally now that there are no more results */
-			phongo_throw_exception_from_bson_error_t_and_reply(&error, doc);
-		}
-	}
-
-	php_phongo_cursor_free_session_if_exhausted(cursor);
-} /* }}} */
-
-static void php_phongo_cursor_iterator_rewind(zend_object_iterator* iter) /* {{{ */
-{
-	php_phongo_cursor_iterator* cursor_it = (php_phongo_cursor_iterator*) iter;
-	php_phongo_cursor_t*        cursor    = cursor_it->cursor;
-	const bson_t*               doc;
-
-	/* If the cursor was never advanced (e.g. command cursor), do so now */
-	if (!cursor->advanced) {
-		cursor->advanced = true;
-
-		if (!phongo_cursor_advance_and_check_for_error(cursor->cursor)) {
-			/* Exception should already have been thrown */
-			return;
-		}
-	}
-
-	if (cursor->current > 0) {
-		phongo_throw_exception(PHONGO_ERROR_LOGIC, "Cursors cannot rewind after starting iteration");
-		return;
-	}
-
-	php_phongo_cursor_free_current(cursor);
-
-	doc = mongoc_cursor_current(cursor->cursor);
-
-	if (doc) {
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data)) {
-			/* Free invalid result, but don't return as we want to free the
-			 * session if the cursor is exhausted. */
-			php_phongo_cursor_free_current(cursor);
-		}
-	}
-
-	php_phongo_cursor_free_session_if_exhausted(cursor);
-} /* }}} */
-
-static zend_object_iterator_funcs php_phongo_cursor_iterator_funcs = {
-	php_phongo_cursor_iterator_dtor,
-	php_phongo_cursor_iterator_valid,
-	php_phongo_cursor_iterator_get_current_data,
-	php_phongo_cursor_iterator_get_current_key,
-	php_phongo_cursor_iterator_move_forward,
-	php_phongo_cursor_iterator_rewind,
-	NULL /* invalidate_current is not used */
-};
-
-static zend_object_iterator* php_phongo_cursor_get_iterator(zend_class_entry* ce, zval* object, int by_ref) /* {{{ */
-{
-	php_phongo_cursor_iterator* cursor_it = NULL;
-	php_phongo_cursor_t*        cursor    = Z_CURSOR_OBJ_P(object);
-
-	if (by_ref) {
-		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
-	}
-
-	if (cursor->got_iterator) {
-		phongo_throw_exception(PHONGO_ERROR_LOGIC, "Cursors cannot yield multiple iterators");
-		return NULL;
-	}
-
-	cursor->got_iterator = true;
-
-	cursor_it = ecalloc(1, sizeof(php_phongo_cursor_iterator));
-	zend_iterator_init(&cursor_it->intern);
-
-	ZVAL_COPY(&cursor_it->intern.data, object);
-	cursor_it->intern.funcs = &php_phongo_cursor_iterator_funcs;
-	cursor_it->cursor       = cursor;
-	/* cursor_it->current should already be allocated to zero */
-
-	php_phongo_cursor_free_current(cursor_it->cursor);
-
-	return &cursor_it->intern;
-} /* }}} */
-/* }}} */
-
 /* {{{ proto void MongoDB\Driver\Cursor::setTypeMap(array $typemap)
    Sets a type map to use for BSON unserialization */
 static PHP_METHOD(Cursor, setTypeMap)
 {
+	zend_error_handling   error_handling;
 	php_phongo_cursor_t*  intern;
 	php_phongo_bson_state state;
 	zval*                 typemap                 = NULL;
@@ -213,9 +66,12 @@ static PHP_METHOD(Cursor, setTypeMap)
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a!", &typemap) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!php_phongo_bson_typemap_to_state(typemap, &state.map)) {
 		return;
@@ -268,17 +124,23 @@ static void php_phongo_cursor_id_new_from_id(zval* object, int64_t cursorid) /*
 
 	object_init_ex(object, php_phongo_cursorid_ce);
 
-	intern     = Z_CURSORID_OBJ_P(object);
-	intern->id = cursorid;
+	intern              = Z_CURSORID_OBJ_P(object);
+	intern->id          = cursorid;
+	intern->initialized = true;
 } /* }}} */
 
 /* {{{ proto array MongoDB\Driver\Cursor::toArray()
    Returns an array of all result documents for this cursor */
 static PHP_METHOD(Cursor, toArray)
 {
+	zend_error_handling error_handling;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	array_init(return_value);
 
@@ -292,13 +154,17 @@ static PHP_METHOD(Cursor, toArray)
    Returns the CursorId for this cursor */
 static PHP_METHOD(Cursor, getId)
 {
+	zend_error_handling  error_handling;
 	php_phongo_cursor_t* intern;
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	php_phongo_cursor_id_new_from_id(return_value, mongoc_cursor_get_id(intern->cursor));
 } /* }}} */
@@ -307,32 +173,183 @@ static PHP_METHOD(Cursor, getId)
    Returns the Server object to which this cursor is attached */
 static PHP_METHOD(Cursor, getServer)
 {
+	zend_error_handling  error_handling;
 	php_phongo_cursor_t* intern;
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	phongo_server_init(return_value, intern->client, intern->server_id);
+	phongo_server_init(return_value, &intern->manager, intern->server_id);
 } /* }}} */
 
 /* {{{ proto boolean MongoDB\Driver\Cursor::isDead()
    Checks if a cursor is still alive */
 static PHP_METHOD(Cursor, isDead)
 {
+	zend_error_handling  error_handling;
 	php_phongo_cursor_t* intern;
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_BOOL(!mongoc_cursor_more(intern->cursor));
 } /* }}} */
 
+PHP_METHOD(Cursor, current)
+{
+	zend_error_handling  error_handling;
+	php_phongo_cursor_t* intern = Z_CURSOR_OBJ_P(getThis());
+	zval*                data;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
+	}
+	zend_restore_error_handling(&error_handling);
+
+	data = &intern->visitor_data.zchild;
+
+	if (Z_ISUNDEF_P(data)) {
+		RETURN_NULL();
+	} else {
+		ZVAL_COPY_DEREF(return_value, data);
+	}
+}
+
+PHP_METHOD(Cursor, key)
+{
+	zend_error_handling  error_handling;
+	php_phongo_cursor_t* intern = Z_CURSOR_OBJ_P(getThis());
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
+	}
+	zend_restore_error_handling(&error_handling);
+
+	if (Z_ISUNDEF(intern->visitor_data.zchild)) {
+		RETURN_NULL();
+	}
+
+	RETURN_LONG(intern->current);
+}
+
+PHP_METHOD(Cursor, next)
+{
+	zend_error_handling  error_handling;
+	php_phongo_cursor_t* intern = Z_CURSOR_OBJ_P(getThis());
+	const bson_t*        doc;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
+	}
+	zend_restore_error_handling(&error_handling);
+
+	php_phongo_cursor_free_current(intern);
+
+	/* If the intern has already advanced, increment its position. Otherwise,
+	 * the first call to mongoc_cursor_next() will be made below and we should
+	 * leave its position at zero. */
+	if (intern->advanced) {
+		intern->current++;
+	} else {
+		intern->advanced = true;
+	}
+
+	if (mongoc_cursor_next(intern->cursor, &doc)) {
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &intern->visitor_data)) {
+			/* Free invalid result, but don't return as we want to free the
+			 * session if the intern is exhausted. */
+			php_phongo_cursor_free_current(intern);
+		}
+	} else {
+		bson_error_t  error = { 0 };
+		const bson_t* doc   = NULL;
+
+		if (mongoc_cursor_error_document(intern->cursor, &error, &doc)) {
+			/* Intentionally not destroying the intern as it will happen
+			 * naturally now that there are no more results */
+			phongo_throw_exception_from_bson_error_t_and_reply(&error, doc);
+		}
+	}
+
+	php_phongo_cursor_free_session_if_exhausted(intern);
+}
+
+PHP_METHOD(Cursor, valid)
+{
+	zend_error_handling  error_handling;
+	php_phongo_cursor_t* intern = Z_CURSOR_OBJ_P(getThis());
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
+	}
+	zend_restore_error_handling(&error_handling);
+
+	RETURN_BOOL(!Z_ISUNDEF(intern->visitor_data.zchild));
+}
+
+PHP_METHOD(Cursor, rewind)
+{
+	zend_error_handling  error_handling;
+	php_phongo_cursor_t* intern = Z_CURSOR_OBJ_P(getThis());
+	const bson_t*        doc;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
+	}
+	zend_restore_error_handling(&error_handling);
+
+	/* If the intern was never advanced (e.g. command intern), do so now */
+	if (!intern->advanced) {
+		intern->advanced = true;
+
+		if (!phongo_cursor_advance_and_check_for_error(intern->cursor)) {
+			/* Exception should already have been thrown */
+			return;
+		}
+	}
+
+	if (intern->current > 0) {
+		phongo_throw_exception(PHONGO_ERROR_LOGIC, "Cursors cannot rewind after starting iteration");
+		return;
+	}
+
+	php_phongo_cursor_free_current(intern);
+
+	doc = mongoc_cursor_current(intern->cursor);
+
+	if (doc) {
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &intern->visitor_data)) {
+			/* Free invalid result, but don't return as we want to free the
+			 * session if the intern is exhausted. */
+			php_phongo_cursor_free_current(intern);
+		}
+	}
+
+	php_phongo_cursor_free_session_if_exhausted(intern);
+}
+
 /* {{{ MongoDB\Driver\Cursor function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_setTypeMap, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, typemap, 0)
@@ -348,6 +365,13 @@ static zend_function_entry php_phongo_cursor_me[] = {
 	PHP_ME(Cursor, getId, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, getServer, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, isDead, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+
+	PHP_ME(Cursor, current, ai_Cursor_void, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, key, ai_Cursor_void, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, next, ai_Cursor_void, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, valid, ai_Cursor_void, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, rewind, ai_Cursor_void, ZEND_ACC_PUBLIC)
+
 	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Cursor_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
@@ -367,7 +391,7 @@ static void php_phongo_cursor_free_object(zend_object* object) /* {{{ */
 	/* If this Cursor was created in a different process, reset the client so
 	 * that mongoc_cursor_destroy does not issue a killCursors command for an
 	 * active cursor owned by a parent process. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
 	if (intern->cursor) {
 		mongoc_cursor_destroy(intern->cursor);
@@ -397,6 +421,10 @@ static void php_phongo_cursor_free_object(zend_object* object) /* {{{ */
 		zval_ptr_dtor(&intern->session);
 	}
 
+	if (!Z_ISUNDEF(intern->manager)) {
+		zval_ptr_dtor(&intern->manager);
+	}
+
 	php_phongo_bson_typemap_dtor(&intern->visitor_data.map);
 
 	php_phongo_cursor_free_current(intern);
@@ -418,13 +446,13 @@ static zend_object* php_phongo_cursor_create_object(zend_class_entry* class_type
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_cursor_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_cursor_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	php_phongo_cursor_t* intern;
 	zval                 retval = ZVAL_STATIC_INIT;
 
 	*is_temp = 1;
-	intern   = Z_CURSOR_OBJ_P(object);
+	intern   = Z_OBJ_CURSOR(PHONGO_COMPAT_GET_OBJ(object));
 
 	array_init_size(&retval, 10);
 
@@ -482,7 +510,7 @@ static HashTable* php_phongo_cursor_get_debug_info(zval* object, int* is_temp) /
 	{
 		zval server;
 
-		phongo_server_init(&server, intern->client, intern->server_id);
+		phongo_server_init(&server, &intern->manager, intern->server_id);
 		ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
 	}
 
@@ -500,8 +528,8 @@ void php_phongo_cursor_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_cursor_ce->create_object = php_phongo_cursor_create_object;
 	PHONGO_CE_FINAL(php_phongo_cursor_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_cursor_ce);
-	php_phongo_cursor_ce->get_iterator = php_phongo_cursor_get_iterator;
 
+	zend_class_implements(php_phongo_cursor_ce, 1, zend_ce_iterator);
 	zend_class_implements(php_phongo_cursor_ce, 1, php_phongo_cursor_interface_ce);
 
 	memcpy(&php_phongo_handler_cursor, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
