@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,9 +27,13 @@
 #include "phongo_compat.h"
 #include "php_phongo.h"
 #include "Session.h"
+#include "src/phongo_apm.h"
 
 #define PHONGO_MANAGER_URI_DEFAULT "mongodb://127.0.0.1/"
 
+#undef MONGOC_LOG_DOMAIN
+#define MONGOC_LOG_DOMAIN "PHONGO"
+
 /**
  * Manager abstracts a cluster of Server objects (i.e. socket connections).
  *
@@ -238,20 +242,19 @@ static bool php_phongo_manager_select_server(bool for_writes, bool inherit_read_
    Constructs a new Manager */
 static PHP_METHOD(Manager, __construct)
 {
-	php_phongo_manager_t* intern;
 	zend_error_handling   error_handling;
+	php_phongo_manager_t* intern;
 	char*                 uri_string     = NULL;
 	size_t                uri_string_len = 0;
 	zval*                 options        = NULL;
 	zval*                 driverOptions  = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	intern = Z_MANAGER_OBJ_P(getThis());
 
 	/* Separate the options and driverOptions zvals, since we may end up
 	 * modifying them in php_phongo_manager_prep_uri_options() and
 	 * php_phongo_manager_merge_context_options() below, respectively. */
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s!a/!a/!", &uri_string, &uri_string_len, &options, &driverOptions) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -269,35 +272,68 @@ static PHP_METHOD(Manager, __construct)
 
 	phongo_manager_init(intern, uri_string ? uri_string : PHONGO_MANAGER_URI_DEFAULT, options, driverOptions);
 
-	if (intern->client) {
-		php_phongo_set_monitoring_callbacks(intern->client);
+	if (EG(exception)) {
+		return;
 	}
+
+	/* Update the request-scoped Manager registry */
+	if (!php_phongo_manager_register(intern)) {
+		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Failed to add Manager to internal registry");
+	}
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\Manager::addSubscriber(MongoDB\Driver\Monitoring\Subscriber $subscriber)
+   Registers an event subscriber for this Manager */
+static PHP_METHOD(Manager, addSubscriber)
+{
+	php_phongo_manager_t* intern;
+	zval*                 subscriber;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_OBJECT_OF_CLASS(subscriber, php_phongo_subscriber_ce)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	intern = Z_MANAGER_OBJ_P(getThis());
+
+	/* Lazily initialize the subscriber HashTable */
+	if (!intern->subscribers) {
+		ALLOC_HASHTABLE(intern->subscribers);
+		zend_hash_init(intern->subscribers, 0, NULL, ZVAL_PTR_DTOR, 0);
+	}
+
+	phongo_apm_add_subscriber(intern->subscribers, subscriber);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\ClientEncryption MongoDB\Driver\Manager::createClientEncryption(array $options)
    Return a ClientEncryption instance */
 static PHP_METHOD(Manager, createClientEncryption)
 {
+	zend_error_handling            error_handling;
 	php_phongo_manager_t*          intern;
 	php_phongo_clientencryption_t* clientencryption;
 	zval*                          options;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
 	object_init_ex(return_value, php_phongo_clientencryption_ce);
 	clientencryption = Z_CLIENTENCRYPTION_OBJ_P(return_value);
 
-	phongo_clientencryption_init(clientencryption, intern->client, options);
+	/* An exception will be thrown on error. */
+	phongo_clientencryption_init(clientencryption, getThis(), options);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
    Execute a Command */
 static PHP_METHOD(Manager, executeCommand)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -308,9 +344,12 @@ static PHP_METHOD(Manager, executeCommand)
 	zval*                 zsession        = NULL;
 	uint32_t              server_id       = 0;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -334,9 +373,9 @@ static PHP_METHOD(Manager, executeCommand)
 	/* If the Manager was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_RAW, db, command, options, server_id, return_value);
+	phongo_execute_command(getThis(), PHONGO_COMMAND_RAW, db, command, options, server_id, return_value);
 
 cleanup:
 	if (free_options) {
@@ -348,6 +387,7 @@ cleanup:
    Execute a ReadCommand */
 static PHP_METHOD(Manager, executeReadCommand)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -357,9 +397,12 @@ static PHP_METHOD(Manager, executeReadCommand)
 	uint32_t              server_id       = 0;
 	zval*                 zsession        = NULL;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -381,15 +424,16 @@ static PHP_METHOD(Manager, executeReadCommand)
 	/* If the Manager was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_READ, db, command, options, server_id, return_value);
+	phongo_execute_command(getThis(), PHONGO_COMMAND_READ, db, command, options, server_id, return_value);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeWriteCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
    Execute a WriteCommand */
 static PHP_METHOD(Manager, executeWriteCommand)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -398,9 +442,12 @@ static PHP_METHOD(Manager, executeWriteCommand)
 	uint32_t              server_id = 0;
 	zval*                 zsession  = NULL;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -417,15 +464,16 @@ static PHP_METHOD(Manager, executeWriteCommand)
 	/* If the Manager was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_WRITE, db, command, options, server_id, return_value);
+	phongo_execute_command(getThis(), PHONGO_COMMAND_WRITE, db, command, options, server_id, return_value);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeReadWriteCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
    Execute a ReadWriteCommand */
 static PHP_METHOD(Manager, executeReadWriteCommand)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -434,9 +482,12 @@ static PHP_METHOD(Manager, executeReadWriteCommand)
 	uint32_t              server_id = 0;
 	zval*                 zsession  = NULL;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -453,15 +504,16 @@ static PHP_METHOD(Manager, executeReadWriteCommand)
 	/* If the Manager was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_READ_WRITE, db, command, options, server_id, return_value);
+	phongo_execute_command(getThis(), PHONGO_COMMAND_READ_WRITE, db, command, options, server_id, return_value);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeQuery(string $namespace, MongoDB\Driver\Query $query[, array $options = null])
    Execute a Query */
 static PHP_METHOD(Manager, executeQuery)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char* namespace;
 	size_t   namespace_len;
@@ -472,9 +524,12 @@ static PHP_METHOD(Manager, executeQuery)
 	uint32_t server_id       = 0;
 	zval*    zsession        = NULL;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &namespace, &namespace_len, &query, php_phongo_query_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -498,9 +553,9 @@ static PHP_METHOD(Manager, executeQuery)
 	/* If the Manager was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
-	phongo_execute_query(intern->client, namespace, query, options, server_id, return_value);
+	phongo_execute_query(getThis(), namespace, query, options, server_id, return_value);
 
 cleanup:
 	if (free_options) {
@@ -512,6 +567,7 @@ cleanup:
    Executes a BulkWrite (i.e. any number of insert, update, and delete ops) */
 static PHP_METHOD(Manager, executeBulkWrite)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char* namespace;
 	size_t                  namespace_len;
@@ -522,9 +578,12 @@ static PHP_METHOD(Manager, executeBulkWrite)
 	uint32_t                server_id    = 0;
 	zval*                   zsession     = NULL;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &namespace, &namespace_len, &zbulk, php_phongo_bulkwrite_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 	bulk   = Z_BULKWRITE_OBJ_P(zbulk);
@@ -543,9 +602,9 @@ static PHP_METHOD(Manager, executeBulkWrite)
 
 	/* If the Server was created in a different process, reset the client so
 	 * that its session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
-	phongo_execute_bulk_write(intern->client, namespace, bulk, options, server_id, return_value);
+	phongo_execute_bulk_write(getThis(), namespace, bulk, options, server_id, return_value);
 
 cleanup:
 	if (free_options) {
@@ -557,13 +616,17 @@ cleanup:
    Returns the ReadConcern associated with this Manager */
 static PHP_METHOD(Manager, getReadConcern)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	phongo_readconcern_init(return_value, mongoc_client_get_read_concern(intern->client));
 } /* }}} */
@@ -572,13 +635,17 @@ static PHP_METHOD(Manager, getReadConcern)
    Returns the ReadPreference associated with this Manager */
 static PHP_METHOD(Manager, getReadPreference)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	phongo_readpreference_init(return_value, mongoc_client_get_read_prefs(intern->client));
 } /* }}} */
@@ -587,15 +654,19 @@ static PHP_METHOD(Manager, getReadPreference)
    Returns the Servers associated with this Manager */
 static PHP_METHOD(Manager, getServers)
 {
+	zend_error_handling           error_handling;
 	php_phongo_manager_t*         intern;
 	mongoc_server_description_t** sds;
 	size_t                        i, n = 0;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	sds = mongoc_client_get_server_descriptions(intern->client, &n);
 	array_init_size(return_value, n);
@@ -603,7 +674,7 @@ static PHP_METHOD(Manager, getServers)
 	for (i = 0; i < n; i++) {
 		zval obj;
 
-		phongo_server_init(&obj, intern->client, mongoc_server_description_id(sds[i]));
+		phongo_server_init(&obj, getThis(), mongoc_server_description_id(sds[i]));
 		add_next_index_zval(return_value, &obj);
 	}
 
@@ -614,43 +685,73 @@ static PHP_METHOD(Manager, getServers)
    Returns the WriteConcern associated with this Manager */
 static PHP_METHOD(Manager, getWriteConcern)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
+	}
+	zend_restore_error_handling(&error_handling);
+
+	phongo_writeconcern_init(return_value, mongoc_client_get_write_concern(intern->client));
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\Manager::removeSubscriber(MongoDB\Driver\Monitoring\Subscriber $subscriber)
+   Unregisters an event subscriber for this Manager */
+static PHP_METHOD(Manager, removeSubscriber)
+{
+	php_phongo_manager_t* intern;
+	zval*                 subscriber;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_OBJECT_OF_CLASS(subscriber, php_phongo_subscriber_ce)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	intern = Z_MANAGER_OBJ_P(getThis());
+
+	/* NOP if subscribers HashTable was never initialized by addSubscriber */
+	if (!intern->subscribers) {
 		return;
 	}
 
-	phongo_writeconcern_init(return_value, mongoc_client_get_write_concern(intern->client));
+	phongo_apm_remove_subscriber(intern->subscribers, subscriber);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Server MongoDB\Driver\Manager::selectServers(MongoDB\Driver\ReadPreference $readPreference)
    Returns a suitable Server for the given ReadPreference */
 static PHP_METHOD(Manager, selectServer)
 {
+	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	zval*                 zreadPreference = NULL;
 	uint32_t              server_id       = 0;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &zreadPreference, php_phongo_readpreference_ce) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!php_phongo_manager_select_server(false, true, zreadPreference, NULL, intern->client, &server_id)) {
 		/* Exception should already have been thrown */
 		return;
 	}
 
-	phongo_server_init(return_value, intern->client, server_id);
+	phongo_server_init(return_value, getThis(), server_id);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Session MongoDB\Driver\Manager::startSession([array $options = null])
    Returns a new client session */
 static PHP_METHOD(Manager, startSession)
 {
+	zend_error_handling       error_handling;
 	php_phongo_manager_t*     intern;
 	zval*                     options = NULL;
 	mongoc_session_opt_t*     cs_opts = NULL;
@@ -660,9 +761,12 @@ static PHP_METHOD(Manager, startSession)
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (options && php_array_existsc(options, "causalConsistency")) {
 		cs_opts = mongoc_session_opts_new();
@@ -704,12 +808,12 @@ static PHP_METHOD(Manager, startSession)
 	/* If the Manager was created in a different process, reset the client so
 	 * that its session pool is cleared. This will ensure that we do not re-use
 	 * a server session (i.e. LSID) created by a parent process. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
 	cs = mongoc_client_start_session(intern->client, cs_opts, &error);
 
 	if (cs) {
-		phongo_session_init(return_value, cs);
+		phongo_session_init(return_value, getThis(), cs);
 	} else {
 		phongo_throw_exception_from_bson_error_t(&error);
 	}
@@ -727,6 +831,10 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Manager___construct, 0, 0, 0)
 	ZEND_ARG_ARRAY_INFO(0, driverOptions, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_Manager_addSubscriber, 0, 0, 1)
+	ZEND_ARG_OBJ_INFO(0, subscriber, MongoDB\\Driver\\Monitoring\\Subscriber, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_Manager_createClientEncryption, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, options, 0)
 ZEND_END_ARG_INFO()
@@ -755,8 +863,12 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Manager_executeBulkWrite, 0, 0, 2)
 	ZEND_ARG_INFO(0, options)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_Manager_removeSubscriber, 0, 0, 1)
+	ZEND_ARG_OBJ_INFO(0, subscriber, MongoDB\\Driver\\Monitoring\\Subscriber, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_Manager_selectServer, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, readPreference, MongoDB\\Driver\\ReadPreference, 1)
+	ZEND_ARG_OBJ_INFO(0, readPreference, MongoDB\\Driver\\ReadPreference, 0)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(ai_Manager_startSession, 0, 0, 0)
@@ -769,6 +881,7 @@ ZEND_END_ARG_INFO()
 static zend_function_entry php_phongo_manager_me[] = {
 	/* clang-format off */
 	PHP_ME(Manager, __construct, ai_Manager___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Manager, addSubscriber, ai_Manager_addSubscriber, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, createClientEncryption, ai_Manager_createClientEncryption, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, executeCommand, ai_Manager_executeCommand, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, executeReadCommand, ai_Manager_executeRWCommand, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -780,6 +893,7 @@ static zend_function_entry php_phongo_manager_me[] = {
 	PHP_ME(Manager, getReadPreference, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, getServers, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, getWriteConcern, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Manager, removeSubscriber, ai_Manager_removeSubscriber, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, selectServer, ai_Manager_selectServer, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, startSession, ai_Manager_startSession, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -797,14 +911,34 @@ static void php_phongo_manager_free_object(zend_object* object) /* {{{ */
 
 	zend_object_std_dtor(&intern->std);
 
+	/* Update the request-scoped Manager registry. The return value is ignored
+	 * because it's possible that the Manager was never registered due to a
+	 * constructor exception. */
+	php_phongo_manager_unregister(intern);
+
 	if (intern->client) {
-		MONGOC_DEBUG("Not destroying persistent client for Manager");
-		intern->client = NULL;
+		/* Request-scoped clients will be removed from the registry and
+		 * destroyed. This is a NOP for persistent clients. The return value is
+		 * ignored because we can't reasonably report an error here. On the off
+		 * chance any request-scoped clients are missed, they will ultimately be
+		 * destroyed in RSHUTDOWN along with the registry HashTable. */
+		php_phongo_client_unregister(intern);
 	}
 
 	if (intern->client_hash) {
 		efree(intern->client_hash);
 	}
+
+	/* Free the keyVaultClient last to ensure that potential non-persistent
+	 * clients are destroyed in the correct order */
+	if (!Z_ISUNDEF(intern->key_vault_client_manager)) {
+		zval_ptr_dtor(&intern->key_vault_client_manager);
+	}
+
+	if (intern->subscribers) {
+		zend_hash_destroy(intern->subscribers);
+		FREE_HASHTABLE(intern->subscribers);
+	}
 } /* }}} */
 
 static zend_object* php_phongo_manager_create_object(zend_class_entry* class_type) /* {{{ */
@@ -823,7 +957,7 @@ static zend_object* php_phongo_manager_create_object(zend_class_entry* class_typ
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_manager_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_manager_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	php_phongo_manager_t*         intern;
 	mongoc_server_description_t** sds;
@@ -832,7 +966,7 @@ static HashTable* php_phongo_manager_get_debug_info(zval* object, int* is_temp)
 	zval                          cluster;
 
 	*is_temp = 1;
-	intern   = Z_MANAGER_OBJ_P(object);
+	intern   = Z_OBJ_MANAGER(PHONGO_COMPAT_GET_OBJ(object));
 
 	array_init_size(&retval, 2);
 
