@@ -1,5 +1,5 @@
 /*
- * Copyright 2016-2017 MongoDB, Inc.
+ * Copyright 2016-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,13 +30,17 @@ zend_class_entry* php_phongo_commandfailedevent_ce;
    Returns the command name for this event */
 PHP_METHOD(CommandFailedEvent, getCommandName)
 {
+	zend_error_handling              error_handling;
 	php_phongo_commandfailedevent_t* intern;
 
 	intern = Z_COMMANDFAILEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETVAL_STRING(intern->command_name);
 } /* }}} */
@@ -45,13 +49,17 @@ PHP_METHOD(CommandFailedEvent, getCommandName)
    Returns the event's duration in microseconds */
 PHP_METHOD(CommandFailedEvent, getDurationMicros)
 {
+	zend_error_handling              error_handling;
 	php_phongo_commandfailedevent_t* intern;
 
 	intern = Z_COMMANDFAILEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_LONG(intern->duration_micros);
 } /* }}} */
@@ -60,13 +68,17 @@ PHP_METHOD(CommandFailedEvent, getDurationMicros)
    Returns the error document associated with the event */
 PHP_METHOD(CommandFailedEvent, getError)
 {
+	zend_error_handling              error_handling;
 	php_phongo_commandfailedevent_t* intern;
 
 	intern = Z_COMMANDFAILEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_ZVAL(&intern->z_error, 1, 0);
 } /* }}} */
@@ -75,14 +87,18 @@ PHP_METHOD(CommandFailedEvent, getError)
    Returns the event's operation ID */
 PHP_METHOD(CommandFailedEvent, getOperationId)
 {
+	zend_error_handling              error_handling;
 	php_phongo_commandfailedevent_t* intern;
 	char                             int_as_string[20];
 
 	intern = Z_COMMANDFAILEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	sprintf(int_as_string, "%" PRIu64, intern->operation_id);
 	RETVAL_STRING(int_as_string);
@@ -92,6 +108,7 @@ PHP_METHOD(CommandFailedEvent, getOperationId)
    Returns the reply document associated with the event */
 PHP_METHOD(CommandFailedEvent, getReply)
 {
+	zend_error_handling              error_handling;
 	php_phongo_commandfailedevent_t* intern;
 	php_phongo_bson_state            state;
 
@@ -99,9 +116,12 @@ PHP_METHOD(CommandFailedEvent, getReply)
 
 	intern = Z_COMMANDFAILEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!php_phongo_bson_to_zval_ex(bson_get_data(intern->reply), intern->reply->len, &state)) {
 		zval_ptr_dtor(&state.zchild);
@@ -115,14 +135,18 @@ PHP_METHOD(CommandFailedEvent, getReply)
    Returns the event's request ID */
 PHP_METHOD(CommandFailedEvent, getRequestId)
 {
+	zend_error_handling              error_handling;
 	php_phongo_commandfailedevent_t* intern;
 	char                             int_as_string[20];
 
 	intern = Z_COMMANDFAILEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	sprintf(int_as_string, "%" PRIu64, intern->request_id);
 	RETVAL_STRING(int_as_string);
@@ -132,15 +156,19 @@ PHP_METHOD(CommandFailedEvent, getRequestId)
    Returns the Server from which the event originated */
 PHP_METHOD(CommandFailedEvent, getServer)
 {
+	zend_error_handling              error_handling;
 	php_phongo_commandfailedevent_t* intern;
 
 	intern = Z_COMMANDFAILEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	phongo_server_init(return_value, intern->client, intern->server_id);
+	phongo_server_init(return_value, &intern->manager, intern->server_id);
 } /* }}} */
 
 /**
@@ -182,6 +210,10 @@ static void php_phongo_commandfailedevent_free_object(zend_object* object) /* {{
 		zval_ptr_dtor(&intern->z_error);
 	}
 
+	if (!Z_ISUNDEF(intern->manager)) {
+		zval_ptr_dtor(&intern->manager);
+	}
+
 	if (intern->reply) {
 		bson_destroy(intern->reply);
 	}
@@ -205,7 +237,7 @@ static zend_object* php_phongo_commandfailedevent_create_object(zend_class_entry
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_commandfailedevent_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_commandfailedevent_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	php_phongo_commandfailedevent_t* intern;
 	zval                             retval = ZVAL_STATIC_INIT;
@@ -214,7 +246,7 @@ static HashTable* php_phongo_commandfailedevent_get_debug_info(zval* object, int
 
 	PHONGO_BSON_INIT_STATE(reply_state);
 
-	intern   = Z_COMMANDFAILEDEVENT_OBJ_P(object);
+	intern   = Z_OBJ_COMMANDFAILEDEVENT(PHONGO_COMPAT_GET_OBJ(object));
 	*is_temp = 1;
 	array_init_size(&retval, 6);
 
@@ -240,7 +272,7 @@ static HashTable* php_phongo_commandfailedevent_get_debug_info(zval* object, int
 	{
 		zval server;
 
-		phongo_server_init(&server, intern->client, intern->server_id);
+		phongo_server_init(&server, &intern->manager, intern->server_id);
 		ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
 	}
 
