@@ -1,5 +1,5 @@
 /*
- * Copyright 2016-2017 MongoDB, Inc.
+ * Copyright 2016-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,13 +30,17 @@ zend_class_entry* php_phongo_commandsucceededevent_ce;
    Returns the command name for this event */
 PHP_METHOD(CommandSucceededEvent, getCommandName)
 {
+	zend_error_handling                 error_handling;
 	php_phongo_commandsucceededevent_t* intern;
 
 	intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETVAL_STRING(intern->command_name);
 } /* }}} */
@@ -45,13 +49,17 @@ PHP_METHOD(CommandSucceededEvent, getCommandName)
    Returns the event's duration in microseconds */
 PHP_METHOD(CommandSucceededEvent, getDurationMicros)
 {
+	zend_error_handling                 error_handling;
 	php_phongo_commandsucceededevent_t* intern;
 
 	intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_LONG(intern->duration_micros);
 } /* }}} */
@@ -60,14 +68,18 @@ PHP_METHOD(CommandSucceededEvent, getDurationMicros)
    Returns the event's operation ID */
 PHP_METHOD(CommandSucceededEvent, getOperationId)
 {
+	zend_error_handling                 error_handling;
 	php_phongo_commandsucceededevent_t* intern;
 	char                                int_as_string[20];
 
 	intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	sprintf(int_as_string, "%" PRIu64, intern->operation_id);
 	RETVAL_STRING(int_as_string);
@@ -77,6 +89,7 @@ PHP_METHOD(CommandSucceededEvent, getOperationId)
    Returns the reply document associated with the event */
 PHP_METHOD(CommandSucceededEvent, getReply)
 {
+	zend_error_handling                 error_handling;
 	php_phongo_commandsucceededevent_t* intern;
 	php_phongo_bson_state               state;
 
@@ -84,9 +97,12 @@ PHP_METHOD(CommandSucceededEvent, getReply)
 
 	intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!php_phongo_bson_to_zval_ex(bson_get_data(intern->reply), intern->reply->len, &state)) {
 		zval_ptr_dtor(&state.zchild);
@@ -100,14 +116,18 @@ PHP_METHOD(CommandSucceededEvent, getReply)
    Returns the event's request ID */
 PHP_METHOD(CommandSucceededEvent, getRequestId)
 {
+	zend_error_handling                 error_handling;
 	php_phongo_commandsucceededevent_t* intern;
 	char                                int_as_string[20];
 
 	intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	sprintf(int_as_string, "%" PRIu64, intern->request_id);
 	RETVAL_STRING(int_as_string);
@@ -117,15 +137,19 @@ PHP_METHOD(CommandSucceededEvent, getRequestId)
    Returns the Server from which the event originated */
 PHP_METHOD(CommandSucceededEvent, getServer)
 {
+	zend_error_handling                 error_handling;
 	php_phongo_commandsucceededevent_t* intern;
 
 	intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	phongo_server_init(return_value, intern->client, intern->server_id);
+	phongo_server_init(return_value, &intern->manager, intern->server_id);
 } /* }}} */
 
 /**
@@ -162,9 +186,14 @@ static void php_phongo_commandsucceededevent_free_object(zend_object* object) /*
 
 	zend_object_std_dtor(&intern->std);
 
+	if (!Z_ISUNDEF(intern->manager)) {
+		zval_ptr_dtor(&intern->manager);
+	}
+
 	if (intern->reply) {
 		bson_destroy(intern->reply);
 	}
+
 	if (intern->command_name) {
 		efree(intern->command_name);
 	}
@@ -184,7 +213,7 @@ static zend_object* php_phongo_commandsucceededevent_create_object(zend_class_en
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_commandsucceededevent_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_commandsucceededevent_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	php_phongo_commandsucceededevent_t* intern;
 	zval                                retval = ZVAL_STATIC_INIT;
@@ -193,7 +222,7 @@ static HashTable* php_phongo_commandsucceededevent_get_debug_info(zval* object,
 
 	PHONGO_BSON_INIT_STATE(reply_state);
 
-	intern   = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(object);
+	intern   = Z_OBJ_COMMANDSUCCEEDEDEVENT(PHONGO_COMPAT_GET_OBJ(object));
 	*is_temp = 1;
 	array_init_size(&retval, 6);
 
@@ -216,7 +245,7 @@ static HashTable* php_phongo_commandsucceededevent_get_debug_info(zval* object,
 	{
 		zval server;
 
-		phongo_server_init(&server, intern->client, intern->server_id);
+		phongo_server_init(&server, &intern->manager, intern->server_id);
 		ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
 	}
 
