@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -63,9 +63,14 @@ static bool php_phongo_readpreference_init_from_hash(php_phongo_readpreference_t
 	}
 
 	if ((tagSets = zend_hash_str_find(props, "tags", sizeof("tags") - 1))) {
+		ZVAL_DEREF(tagSets);
 		if (Z_TYPE_P(tagSets) == IS_ARRAY) {
 			bson_t* tags = bson_new();
 
+			/* Separate tagSets as php_phongo_read_preference_prep_tagsets may
+			 * modify these tags. */
+			SEPARATE_ZVAL_NOREF(tagSets);
+
 			php_phongo_read_preference_prep_tagsets(tagSets);
 			php_phongo_zval_to_bson(tagSets, PHONGO_BSON_NONE, (bson_t*) tags, NULL);
 
@@ -177,17 +182,17 @@ static const char* php_phongo_readpreference_get_mode_string(mongoc_read_mode_t
    Constructs a new ReadPreference */
 static PHP_METHOD(ReadPreference, __construct)
 {
-	php_phongo_readpreference_t* intern;
 	zend_error_handling          error_handling;
+	php_phongo_readpreference_t* intern;
 	zval*                        mode;
 	zval*                        tagSets = NULL;
 	zval*                        options = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
 	/* Separate the tagSets zval, since we may end up modifying it in
 	 * php_phongo_read_preference_prep_tagsets() below. */
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|a/!a!", &mode, &tagSets, &options) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -303,17 +308,24 @@ static PHP_METHOD(ReadPreference, __construct)
 	}
 } /* }}} */
 
-/* {{{ proto void MongoDB\BSON\ReadPreference::__set_state(array $properties)
+/* {{{ proto MongoDB\Driver\ReadPreference MongoDB\Driver\ReadPreference::__set_state(array $properties)
 */
 static PHP_METHOD(ReadPreference, __set_state)
 {
+	zend_error_handling          error_handling;
 	php_phongo_readpreference_t* intern;
 	HashTable*                   props;
 	zval*                        array;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+	/* Separate the zval, since we may end up modifying the "tags" element in
+	 * php_phongo_read_preference_prep_tagsets(), which is called from
+	 * php_phongo_readpreference_init_from_hash. */
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a/", &array) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_readpreference_ce);
 
@@ -327,14 +339,18 @@ static PHP_METHOD(ReadPreference, __set_state)
    Returns the ReadPreference hedge document */
 static PHP_METHOD(ReadPreference, getHedge)
 {
+	zend_error_handling          error_handling;
 	php_phongo_readpreference_t* intern;
 	const bson_t*                hedge;
 
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	hedge = mongoc_read_prefs_get_hedge(intern->read_preference);
 
@@ -358,13 +374,17 @@ static PHP_METHOD(ReadPreference, getHedge)
    Returns the ReadPreference maxStalenessSeconds value */
 static PHP_METHOD(ReadPreference, getMaxStalenessSeconds)
 {
+	zend_error_handling          error_handling;
 	php_phongo_readpreference_t* intern;
 
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_LONG(mongoc_read_prefs_get_max_staleness_seconds(intern->read_preference));
 } /* }}} */
@@ -373,13 +393,17 @@ static PHP_METHOD(ReadPreference, getMaxStalenessSeconds)
    Returns the ReadPreference mode */
 static PHP_METHOD(ReadPreference, getMode)
 {
+	zend_error_handling          error_handling;
 	php_phongo_readpreference_t* intern;
 
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_LONG(mongoc_read_prefs_get_mode(intern->read_preference));
 } /* }}} */
@@ -388,14 +412,18 @@ static PHP_METHOD(ReadPreference, getMode)
    Returns the ReadPreference mode as string */
 static PHP_METHOD(ReadPreference, getModeString)
 {
+	zend_error_handling          error_handling;
 	php_phongo_readpreference_t* intern;
 	const char*                  mode_string;
 
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	mode_string = php_phongo_readpreference_get_mode_string(mongoc_read_prefs_get_mode(intern->read_preference));
 	if (!mode_string) {
@@ -410,14 +438,18 @@ static PHP_METHOD(ReadPreference, getModeString)
    Returns the ReadPreference tag sets */
 static PHP_METHOD(ReadPreference, getTagSets)
 {
+	zend_error_handling          error_handling;
 	php_phongo_readpreference_t* intern;
 	const bson_t*                tags;
 
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	tags = mongoc_read_prefs_get_tags(intern->read_preference);
 
@@ -437,7 +469,7 @@ static PHP_METHOD(ReadPreference, getTagSets)
 	}
 } /* }}} */
 
-static HashTable* php_phongo_readpreference_get_properties_hash(zval* object, bool is_debug) /* {{{ */
+static HashTable* php_phongo_readpreference_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
 {
 	php_phongo_readpreference_t* intern;
 	HashTable*                   props;
@@ -446,7 +478,7 @@ static HashTable* php_phongo_readpreference_get_properties_hash(zval* object, bo
 	const bson_t*                hedge;
 	mongoc_read_mode_t           mode;
 
-	intern = Z_READPREFERENCE_OBJ_P(object);
+	intern = Z_OBJ_READPREFERENCE(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 4);
 
@@ -511,11 +543,16 @@ done:
 */
 static PHP_METHOD(ReadPreference, bsonSerialize)
 {
+	zend_error_handling error_handling;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	ZVAL_ARR(return_value, php_phongo_readpreference_get_properties_hash(getThis(), true));
+	ZVAL_ARR(return_value, php_phongo_readpreference_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
 	convert_to_object(return_value);
 } /* }}} */
 
@@ -523,6 +560,7 @@ static PHP_METHOD(ReadPreference, bsonSerialize)
 */
 static PHP_METHOD(ReadPreference, serialize)
 {
+	zend_error_handling          error_handling;
 	php_phongo_readpreference_t* intern;
 	zval                         retval;
 	php_serialize_data_t         var_hash;
@@ -535,9 +573,12 @@ static PHP_METHOD(ReadPreference, serialize)
 
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!intern->read_preference) {
 		return;
@@ -600,8 +641,8 @@ static PHP_METHOD(ReadPreference, serialize)
 */
 static PHP_METHOD(ReadPreference, unserialize)
 {
-	php_phongo_readpreference_t* intern;
 	zend_error_handling          error_handling;
+	php_phongo_readpreference_t* intern;
 	char*                        serialized;
 	size_t                       serialized_len;
 	zval                         props;
@@ -610,7 +651,6 @@ static PHP_METHOD(ReadPreference, unserialize)
 	intern = Z_READPREFERENCE_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -703,13 +743,13 @@ static zend_object* php_phongo_readpreference_create_object(zend_class_entry* cl
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_readpreference_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_readpreference_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_readpreference_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_readpreference_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_readpreference_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_readpreference_get_properties_hash(object, false);
 } /* }}} */
