@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,6 +31,7 @@ zend_class_entry* php_phongo_server_ce;
    Executes a Command on this Server */
 static PHP_METHOD(Server, executeCommand)
 {
+	zend_error_handling  error_handling;
 	php_phongo_server_t* intern;
 	char*                db;
 	size_t               db_len;
@@ -40,18 +41,21 @@ static PHP_METHOD(Server, executeCommand)
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	options = php_phongo_prep_legacy_option(options, "readPreference", &free_options);
 
 	/* If the Server was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_RAW, db, command, options, intern->server_id, return_value);
+	phongo_execute_command(&intern->manager, PHONGO_COMMAND_RAW, db, command, options, intern->server_id, return_value);
 
 	if (free_options) {
 		php_phongo_prep_legacy_option_free(options);
@@ -62,6 +66,7 @@ static PHP_METHOD(Server, executeCommand)
    Executes a ReadCommand on this Server */
 static PHP_METHOD(Server, executeReadCommand)
 {
+	zend_error_handling  error_handling;
 	php_phongo_server_t* intern;
 	char*                db;
 	size_t               db_len;
@@ -70,22 +75,26 @@ static PHP_METHOD(Server, executeReadCommand)
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	/* If the Server was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_READ, db, command, options, intern->server_id, return_value);
+	phongo_execute_command(&intern->manager, PHONGO_COMMAND_READ, db, command, options, intern->server_id, return_value);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Server::executeWriteCommand(string $db, MongoDB\Driver\Command $command[, array $options = null]))
    Executes a WriteCommand on this Server */
 static PHP_METHOD(Server, executeWriteCommand)
 {
+	zend_error_handling  error_handling;
 	php_phongo_server_t* intern;
 	char*                db;
 	size_t               db_len;
@@ -94,22 +103,26 @@ static PHP_METHOD(Server, executeWriteCommand)
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	/* If the Server was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated. and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_WRITE, db, command, options, intern->server_id, return_value);
+	phongo_execute_command(&intern->manager, PHONGO_COMMAND_WRITE, db, command, options, intern->server_id, return_value);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Server::executeReadWriteCommand(string $db, MongoDB\Driver\Command $command[, array $options = null]))
    Executes a ReadWriteCommand on this Server */
 static PHP_METHOD(Server, executeReadWriteCommand)
 {
+	zend_error_handling  error_handling;
 	php_phongo_server_t* intern;
 	char*                db;
 	size_t               db_len;
@@ -118,22 +131,26 @@ static PHP_METHOD(Server, executeReadWriteCommand)
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	/* If the Server was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
-	phongo_execute_command(intern->client, PHONGO_COMMAND_READ_WRITE, db, command, options, intern->server_id, return_value);
+	phongo_execute_command(&intern->manager, PHONGO_COMMAND_READ_WRITE, db, command, options, intern->server_id, return_value);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Server::executeQuery(string $namespace, MongoDB\Driver\Query $query[, array $options = null]))
    Executes a Query on this Server */
 static PHP_METHOD(Server, executeQuery)
 {
+	zend_error_handling  error_handling;
 	php_phongo_server_t* intern;
 	char* namespace;
 	size_t namespace_len;
@@ -143,18 +160,21 @@ static PHP_METHOD(Server, executeQuery)
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &namespace, &namespace_len, &query, php_phongo_query_ce, &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	options = php_phongo_prep_legacy_option(options, "readPreference", &free_options);
 
 	/* If the Server was created in a different process, reset the client so
 	 * that cursors created by this process can be differentiated and its
 	 * session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
-	phongo_execute_query(intern->client, namespace, query, options, intern->server_id, return_value);
+	phongo_execute_query(&intern->manager, namespace, query, options, intern->server_id, return_value);
 
 	if (free_options) {
 		php_phongo_prep_legacy_option_free(options);
@@ -166,6 +186,7 @@ static PHP_METHOD(Server, executeQuery)
    this Server */
 static PHP_METHOD(Server, executeBulkWrite)
 {
+	zend_error_handling  error_handling;
 	php_phongo_server_t* intern;
 	char* namespace;
 	size_t                  namespace_len;
@@ -176,9 +197,12 @@ static PHP_METHOD(Server, executeBulkWrite)
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &namespace, &namespace_len, &zbulk, php_phongo_bulkwrite_ce, &options, php_phongo_writeconcern_ce) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	bulk = Z_BULKWRITE_OBJ_P(zbulk);
 
@@ -186,9 +210,9 @@ static PHP_METHOD(Server, executeBulkWrite)
 
 	/* If the Server was created in a different process, reset the client so
 	 * that its session pool is cleared. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
-	phongo_execute_bulk_write(intern->client, namespace, bulk, options, intern->server_id, return_value);
+	phongo_execute_bulk_write(&intern->manager, namespace, bulk, options, intern->server_id, return_value);
 
 	if (free_options) {
 		php_phongo_prep_legacy_option_free(options);
@@ -199,16 +223,20 @@ static PHP_METHOD(Server, executeBulkWrite)
    Returns the hostname for this Server */
 static PHP_METHOD(Server, getHost)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		RETVAL_STRING(mongoc_server_description_host(sd)->host);
 		mongoc_server_description_destroy(sd);
 		return;
@@ -221,20 +249,24 @@ static PHP_METHOD(Server, getHost)
    Returns the currently configured tags for this Server */
 static PHP_METHOD(Server, getTags)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
-		const bson_t* is_master = mongoc_server_description_ismaster(sd);
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
+		const bson_t* hello_response = mongoc_server_description_hello_response(sd);
 		bson_iter_t   iter;
 
-		if (bson_iter_init_find(&iter, is_master, "tags") && BSON_ITER_HOLDS_DOCUMENT(&iter)) {
+		if (bson_iter_init_find(&iter, hello_response, "tags") && BSON_ITER_HOLDS_DOCUMENT(&iter)) {
 			const uint8_t*        bytes;
 			uint32_t              len;
 			php_phongo_bson_state state;
@@ -263,25 +295,29 @@ static PHP_METHOD(Server, getTags)
 } /* }}} */
 
 /* {{{ proto array MongoDB\Driver\Server::getInfo()
-   Returns the last isMaster result document for this Server */
+   Returns the last hello response for this Server */
 static PHP_METHOD(Server, getInfo)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
-		const bson_t*         is_master = mongoc_server_description_ismaster(sd);
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
+		const bson_t*         hello_response = mongoc_server_description_hello_response(sd);
 		php_phongo_bson_state state;
 
 		PHONGO_BSON_INIT_DEBUG_STATE(state);
 
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(is_master), is_master->len, &state)) {
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(hello_response), hello_response->len, &state)) {
 			/* Exception should already have been thrown */
 			zval_ptr_dtor(&state.zchild);
 			mongoc_server_description_destroy(sd);
@@ -300,16 +336,20 @@ static PHP_METHOD(Server, getInfo)
    Returns the last measured latency for this Server */
 static PHP_METHOD(Server, getLatency)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		RETVAL_LONG((zend_long) mongoc_server_description_round_trip_time(sd));
 		mongoc_server_description_destroy(sd);
 		return;
@@ -322,16 +362,20 @@ static PHP_METHOD(Server, getLatency)
    Returns the port for this Server */
 static PHP_METHOD(Server, getPort)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		RETVAL_LONG(mongoc_server_description_host(sd)->port);
 		mongoc_server_description_destroy(sd);
 		return;
@@ -344,16 +388,20 @@ static PHP_METHOD(Server, getPort)
    Returns the node type of this Server */
 static PHP_METHOD(Server, getType)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		RETVAL_LONG(php_phongo_server_description_type(sd));
 		mongoc_server_description_destroy(sd);
 		return;
@@ -366,16 +414,20 @@ static PHP_METHOD(Server, getType)
    Returns whether this Server is a primary member of a replica set */
 static PHP_METHOD(Server, isPrimary)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		RETVAL_BOOL(!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_PRIMARY].name));
 		mongoc_server_description_destroy(sd);
 		return;
@@ -388,16 +440,20 @@ static PHP_METHOD(Server, isPrimary)
    Returns whether this Server is a secondary member of a replica set */
 static PHP_METHOD(Server, isSecondary)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		RETVAL_BOOL(!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_SECONDARY].name));
 		mongoc_server_description_destroy(sd);
 		return;
@@ -410,16 +466,20 @@ static PHP_METHOD(Server, isSecondary)
    Returns whether this Server is an arbiter member of a replica set */
 static PHP_METHOD(Server, isArbiter)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		RETVAL_BOOL(!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_ARBITER].name));
 		mongoc_server_description_destroy(sd);
 		return;
@@ -432,19 +492,23 @@ static PHP_METHOD(Server, isArbiter)
    Returns whether this Server is a hidden member of a replica set */
 static PHP_METHOD(Server, isHidden)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		bson_iter_t iter;
 
-		RETVAL_BOOL(bson_iter_init_find_case(&iter, mongoc_server_description_ismaster(sd), "hidden") && bson_iter_as_bool(&iter));
+		RETVAL_BOOL(bson_iter_init_find_case(&iter, mongoc_server_description_hello_response(sd), "hidden") && bson_iter_as_bool(&iter));
 		mongoc_server_description_destroy(sd);
 		return;
 	}
@@ -456,19 +520,23 @@ static PHP_METHOD(Server, isHidden)
    Returns whether this Server is a passive member of a replica set */
 static PHP_METHOD(Server, isPassive)
 {
+	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		bson_iter_t iter;
 
-		RETVAL_BOOL(bson_iter_init_find_case(&iter, mongoc_server_description_ismaster(sd), "passive") && bson_iter_as_bool(&iter));
+		RETVAL_BOOL(bson_iter_init_find_case(&iter, mongoc_server_description_hello_response(sd), "passive") && bson_iter_as_bool(&iter));
 		mongoc_server_description_destroy(sd);
 		return;
 	}
@@ -540,11 +608,13 @@ static int php_phongo_server_compare_objects(zval* o1, zval* o2) /* {{{ */
 	mongoc_server_description_t *sd1, *sd2;
 	int                          retval = 0;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_SERVER_OBJ_P(o1);
 	intern2 = Z_SERVER_OBJ_P(o2);
 
-	sd1 = mongoc_client_get_server_description(intern1->client, intern1->server_id);
-	sd2 = mongoc_client_get_server_description(intern2->client, intern2->server_id);
+	sd1 = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern1->manager)->client, intern1->server_id);
+	sd2 = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern2->manager)->client, intern2->server_id);
 
 	if (sd1 && sd2) {
 		retval = strcasecmp(mongoc_server_description_host(sd1)->host_and_port, mongoc_server_description_host(sd2)->host_and_port);
@@ -568,6 +638,10 @@ static void php_phongo_server_free_object(zend_object* object) /* {{{ */
 	php_phongo_server_t* intern = Z_OBJ_SERVER(object);
 
 	zend_object_std_dtor(&intern->std);
+
+	if (!Z_ISUNDEF(intern->manager)) {
+		zval_ptr_dtor(&intern->manager);
+	}
 } /* }}} */
 
 static zend_object* php_phongo_server_create_object(zend_class_entry* class_type) /* {{{ */
@@ -586,16 +660,16 @@ static zend_object* php_phongo_server_create_object(zend_class_entry* class_type
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_server_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_server_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	php_phongo_server_t*         intern = NULL;
 	zval                         retval = ZVAL_STATIC_INIT;
 	mongoc_server_description_t* sd;
 
 	*is_temp = 1;
-	intern   = Z_SERVER_OBJ_P(object);
+	intern   = Z_OBJ_SERVER(PHONGO_COMPAT_GET_OBJ(object));
 
-	if (!(sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+	if (!(sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
 		phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get server description");
 		return NULL;
 	}
@@ -618,10 +692,10 @@ void php_phongo_server_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_server_ce);
 
 	memcpy(&php_phongo_handler_server, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_server.compare_objects = php_phongo_server_compare_objects;
-	php_phongo_handler_server.get_debug_info  = php_phongo_server_get_debug_info;
-	php_phongo_handler_server.free_obj        = php_phongo_server_free_object;
-	php_phongo_handler_server.offset          = XtOffsetOf(php_phongo_server_t, std);
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(server);
+	php_phongo_handler_server.get_debug_info = php_phongo_server_get_debug_info;
+	php_phongo_handler_server.free_obj       = php_phongo_server_free_object;
+	php_phongo_handler_server.offset         = XtOffsetOf(php_phongo_server_t, std);
 
 	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_UNKNOWN"), PHONGO_SERVER_UNKNOWN);
 	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_STANDALONE"), PHONGO_SERVER_STANDALONE);
