@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2017-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -50,13 +50,13 @@ static bool php_phongo_session_get_timestamp_parts(zval* obj, uint32_t* timestam
 	zval ztimestamp = ZVAL_STATIC_INIT;
 	zval zincrement = ZVAL_STATIC_INIT;
 
-	zend_call_method_with_0_params(obj, NULL, NULL, "getTimestamp", &ztimestamp);
+	zend_call_method_with_0_params(PHONGO_COMPAT_OBJ_P(obj), NULL, NULL, "getTimestamp", &ztimestamp);
 
 	if (Z_ISUNDEF(ztimestamp) || EG(exception)) {
 		goto cleanup;
 	}
 
-	zend_call_method_with_0_params(obj, NULL, NULL, "getIncrement", &zincrement);
+	zend_call_method_with_0_params(PHONGO_COMPAT_OBJ_P(obj), NULL, NULL, "getIncrement", &zincrement);
 
 	if (Z_ISUNDEF(zincrement) || EG(exception)) {
 		goto cleanup;
@@ -98,10 +98,66 @@ static const char* php_phongo_get_transaction_state_string(mongoc_transaction_st
 	}
 }
 
+static void php_phongo_transaction_options_to_zval(mongoc_client_session_t* cs, zval* retval)
+{
+	mongoc_transaction_opt_t*     opts;
+	int64_t                       max_commit_time_ms;
+	const mongoc_read_concern_t*  read_concern;
+	const mongoc_read_prefs_t*    read_preference;
+	const mongoc_write_concern_t* write_concern;
+
+	if (!cs) {
+		ZVAL_NULL(retval);
+		return;
+	}
+
+	opts = mongoc_session_opts_get_transaction_opts(cs);
+
+	if (!opts) {
+		ZVAL_NULL(retval);
+		return;
+	}
+
+	max_commit_time_ms = mongoc_transaction_opts_get_max_commit_time_ms(opts);
+	read_concern       = mongoc_transaction_opts_get_read_concern(opts);
+	read_preference    = mongoc_transaction_opts_get_read_prefs(opts);
+	write_concern      = mongoc_transaction_opts_get_write_concern(opts);
+
+	array_init_size(retval, 4);
+
+	if (max_commit_time_ms) {
+		ADD_ASSOC_LONG_EX(retval, "maxCommitTimeMS", max_commit_time_ms);
+	}
+
+	if (!mongoc_read_concern_is_default(read_concern)) {
+		zval zread_concern;
+
+		phongo_readconcern_init(&zread_concern, read_concern);
+		ADD_ASSOC_ZVAL_EX(retval, "readConcern", &zread_concern);
+	}
+
+	if (read_preference) {
+		zval zread_preference;
+
+		phongo_readpreference_init(&zread_preference, read_preference);
+		ADD_ASSOC_ZVAL_EX(retval, "readPreference", &zread_preference);
+	}
+
+	if (!mongoc_write_concern_is_default(write_concern)) {
+		zval zwrite_concern;
+
+		phongo_writeconcern_init(&zwrite_concern, write_concern);
+		ADD_ASSOC_ZVAL_EX(retval, "writeConcern", &zwrite_concern);
+	}
+
+	mongoc_transaction_opts_destroy(opts);
+}
+
 /* {{{ proto void MongoDB\Driver\Session::advanceClusterTime(array|object $clusterTime)
    Advances the cluster time for this Session */
 static PHP_METHOD(Session, advanceClusterTime)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	zval*                 zcluster_time;
 	bson_t                cluster_time = BSON_INITIALIZER;
@@ -109,9 +165,12 @@ static PHP_METHOD(Session, advanceClusterTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceClusterTime")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A", &zcluster_time) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	php_phongo_zval_to_bson(zcluster_time, PHONGO_BSON_NONE, &cluster_time, NULL);
 
@@ -130,6 +189,7 @@ cleanup:
    Advances the operation time for this Session */
 static PHP_METHOD(Session, advanceOperationTime)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	zval*                 ztimestamp;
 	uint32_t              timestamp = 0;
@@ -138,9 +198,12 @@ static PHP_METHOD(Session, advanceOperationTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceOperationTime")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &ztimestamp, php_phongo_timestamp_interface_ce) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!php_phongo_session_get_timestamp_parts(ztimestamp, &timestamp, &increment)) {
 		return;
@@ -153,6 +216,7 @@ static PHP_METHOD(Session, advanceOperationTime)
    Returns the cluster time for this Session */
 static PHP_METHOD(Session, getClusterTime)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const bson_t*         cluster_time;
 	php_phongo_bson_state state;
@@ -162,9 +226,12 @@ static PHP_METHOD(Session, getClusterTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getClusterTime")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
 
@@ -185,6 +252,7 @@ static PHP_METHOD(Session, getClusterTime)
    Returns the logical session ID for this Session */
 static PHP_METHOD(Session, getLogicalSessionId)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const bson_t*         lsid;
 	php_phongo_bson_state state;
@@ -194,9 +262,12 @@ static PHP_METHOD(Session, getLogicalSessionId)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getLogicalSessionId")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	lsid = mongoc_client_session_get_lsid(intern->client_session);
 
@@ -213,15 +284,19 @@ static PHP_METHOD(Session, getLogicalSessionId)
    Returns the operation time for this Session */
 static PHP_METHOD(Session, getOperationTime)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	uint32_t              timestamp, increment;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getOperationTime")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	mongoc_client_session_get_operation_time(intern->client_session, &timestamp, &increment);
 
@@ -239,15 +314,19 @@ static PHP_METHOD(Session, getOperationTime)
    Returns the server this session is pinned to */
 static PHP_METHOD(Session, getServer)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	uint32_t              server_id = 0;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getServer")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	server_id = mongoc_client_session_get_server_id(intern->client_session);
 
@@ -256,79 +335,46 @@ static PHP_METHOD(Session, getServer)
 		RETURN_NULL();
 	}
 
-	phongo_server_init(return_value, intern->client, server_id);
+	phongo_server_init(return_value, &intern->manager, server_id);
 } /* }}} */
 
 /* {{{ proto array|null MongoDB\Driver\Session::getTransactionOptions()
    Returns options for the currently running transaction */
 static PHP_METHOD(Session, getTransactionOptions)
 {
-	php_phongo_session_t*         intern;
-	mongoc_transaction_opt_t*     opts;
-	int64_t                       max_commit_time_ms;
-	const mongoc_read_concern_t*  read_concern;
-	const mongoc_read_prefs_t*    read_preference;
-	const mongoc_write_concern_t* write_concern;
+	zend_error_handling   error_handling;
+	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getTransactionOptions")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	opts = mongoc_session_opts_get_transaction_opts(intern->client_session);
-
-	if (!opts) {
-		return;
-	}
-
-	max_commit_time_ms = mongoc_transaction_opts_get_max_commit_time_ms(opts);
-	read_concern       = mongoc_transaction_opts_get_read_concern(opts);
-	read_preference    = mongoc_transaction_opts_get_read_prefs(opts);
-	write_concern      = mongoc_transaction_opts_get_write_concern(opts);
-
-	array_init_size(return_value, 4);
-
-	if (max_commit_time_ms) {
-		ADD_ASSOC_LONG_EX(return_value, "maxCommitTimeMS", max_commit_time_ms);
-	}
-
-	if (!mongoc_read_concern_is_default(read_concern)) {
-		zval zread_concern;
-
-		phongo_readconcern_init(&zread_concern, read_concern);
-		ADD_ASSOC_ZVAL_EX(return_value, "readConcern", &zread_concern);
-	}
-
-	if (read_preference) {
-		zval zread_preference;
-
-		phongo_readpreference_init(&zread_preference, read_preference);
-		ADD_ASSOC_ZVAL_EX(return_value, "readPreference", &zread_preference);
-	}
-
-	if (!mongoc_write_concern_is_default(write_concern)) {
-		zval zwrite_concern;
-
-		phongo_writeconcern_init(&zwrite_concern, write_concern);
-		ADD_ASSOC_ZVAL_EX(return_value, "writeConcern", &zwrite_concern);
-	}
+	php_phongo_transaction_options_to_zval(intern->client_session, return_value);
 } /* }}} */
 
 /* {{{ proto string MongoDB\Driver\Session::getTransactionState()
    Returns the current transaction state for this session */
 static PHP_METHOD(Session, getTransactionState)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const char*           state;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getTransactionState")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	state = php_phongo_get_transaction_state_string(mongoc_client_session_get_transaction_state(intern->client_session));
 	if (!state) {
@@ -440,6 +486,7 @@ mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* op
    Starts a new transaction */
 static PHP_METHOD(Session, startTransaction)
 {
+	zend_error_handling       error_handling;
 	php_phongo_session_t*     intern;
 	zval*                     options     = NULL;
 	mongoc_transaction_opt_t* txn_options = NULL;
@@ -448,9 +495,12 @@ static PHP_METHOD(Session, startTransaction)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "startTransaction")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (options) {
 		txn_options = php_mongodb_session_parse_transaction_options(options);
@@ -472,6 +522,7 @@ static PHP_METHOD(Session, startTransaction)
    Commits an existing transaction */
 static PHP_METHOD(Session, commitTransaction)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	bson_error_t          error;
 	bson_t                reply;
@@ -479,9 +530,12 @@ static PHP_METHOD(Session, commitTransaction)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "commitTransaction")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!mongoc_client_session_commit_transaction(intern->client_session, &reply, &error)) {
 		phongo_throw_exception_from_bson_error_t_and_reply(&error, &reply);
@@ -494,15 +548,19 @@ static PHP_METHOD(Session, commitTransaction)
    Aborts (rolls back) an existing transaction */
 static PHP_METHOD(Session, abortTransaction)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	bson_error_t          error;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "abortTransaction")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!mongoc_client_session_abort_transaction(intern->client_session, &error)) {
 		phongo_throw_exception_from_bson_error_t(&error);
@@ -513,13 +571,17 @@ static PHP_METHOD(Session, abortTransaction)
    Ends the session, and a running transaction if active */
 static PHP_METHOD(Session, endSession)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	mongoc_client_session_destroy(intern->client_session);
 	intern->client_session = NULL;
@@ -529,14 +591,18 @@ static PHP_METHOD(Session, endSession)
    Returns whether a multi-document transaction is in progress */
 static PHP_METHOD(Session, isInTransaction)
 {
+	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "isInTransaction")
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_BOOL(mongoc_client_session_in_transaction(intern->client_session));
 } /* }}} */
@@ -593,11 +659,15 @@ static void php_phongo_session_free_object(zend_object* object) /* {{{ */
 	 * destroy the corresponding server session rather than return it to the
 	 * now-empty pool. This will ensure that we do not re-use a server session
 	 * (i.e. LSID) created by a parent process. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
 	if (intern->client_session) {
 		mongoc_client_session_destroy(intern->client_session);
 	}
+
+	if (!Z_ISUNDEF(intern->manager)) {
+		zval_ptr_dtor(&intern->manager);
+	}
 } /* }}} */
 
 static zend_object* php_phongo_session_create_object(zend_class_entry* class_type) /* {{{ */
@@ -616,14 +686,13 @@ static zend_object* php_phongo_session_create_object(zend_class_entry* class_typ
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
-	php_phongo_session_t*       intern = NULL;
-	const mongoc_session_opt_t* cs_opts;
-	zval                        retval = ZVAL_STATIC_INIT;
+	php_phongo_session_t* intern = NULL;
+	zval                  retval = ZVAL_STATIC_INIT;
 
 	*is_temp = 1;
-	intern   = Z_SESSION_OBJ_P(object);
+	intern   = Z_OBJ_SESSION(PHONGO_COMPAT_GET_OBJ(object));
 
 	array_init(&retval);
 
@@ -669,7 +738,7 @@ static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp)
 	}
 
 	if (intern->client_session) {
-		cs_opts = mongoc_client_session_get_opts(intern->client_session);
+		const mongoc_session_opt_t* cs_opts = mongoc_client_session_get_opts(intern->client_session);
 		ADD_ASSOC_BOOL_EX(&retval, "causalConsistency", mongoc_session_opts_get_causal_consistency(cs_opts));
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "causalConsistency");
@@ -699,7 +768,7 @@ static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp)
 
 			zval server;
 
-			phongo_server_init(&server, intern->client, server_id);
+			phongo_server_init(&server, &intern->manager, server_id);
 			ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "server");
@@ -708,6 +777,33 @@ static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp)
 		ADD_ASSOC_NULL_EX(&retval, "server");
 	}
 
+	if (intern->client_session) {
+		ADD_ASSOC_BOOL_EX(&retval, "inTransaction", mongoc_client_session_in_transaction(intern->client_session));
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "inTransaction");
+	}
+
+	if (intern->client_session) {
+		const char* state = php_phongo_get_transaction_state_string(mongoc_client_session_get_transaction_state(intern->client_session));
+
+		if (!state) {
+			/* Exception should already have been thrown */
+			goto done;
+		}
+
+		ADD_ASSOC_STRING(&retval, "transactionState", state);
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "transactionState");
+	}
+
+	if (intern->client_session) {
+		zval txn_opts;
+		php_phongo_transaction_options_to_zval(intern->client_session, &txn_opts);
+		ADD_ASSOC_ZVAL_EX(&retval, "transactionOptions", &txn_opts);
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "transactionOptions");
+	}
+
 done:
 	return Z_ARRVAL(retval);
 } /* }}} */
