@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -109,14 +109,14 @@ failure:
    Constructs a new WriteConcern */
 static PHP_METHOD(WriteConcern, __construct)
 {
-	php_phongo_writeconcern_t* intern;
 	zend_error_handling        error_handling;
+	php_phongo_writeconcern_t* intern;
 	zval *                     w, *journal;
 	zend_long                  wtimeout = 0;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|lz", &w, &wtimeout, &journal) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -152,7 +152,8 @@ static PHP_METHOD(WriteConcern, __construct)
 
 				mongoc_write_concern_set_journal(intern->write_concern, zend_is_true(journal));
 			}
-			/* fallthrough */
+			PHONGO_BREAK_INTENTIONALLY_MISSING
+
 		case 2:
 			if (wtimeout < 0) {
 				phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected wtimeout to be >= 0, %" PHONGO_LONG_FORMAT " given", wtimeout);
@@ -168,17 +169,21 @@ static PHP_METHOD(WriteConcern, __construct)
 	}
 } /* }}} */
 
-/* {{{ proto void MongoDB\BSON\WriteConcern::__set_state(array $properties)
+/* {{{ proto MongoDB\Driver\WriteConcern MongoDB\Driver\WriteConcern::__set_state(array $properties)
 */
 static PHP_METHOD(WriteConcern, __set_state)
 {
+	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	HashTable*                 props;
 	zval*                      array;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		RETURN_FALSE;
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_writeconcern_ce);
 
@@ -192,14 +197,18 @@ static PHP_METHOD(WriteConcern, __set_state)
    Returns the WriteConcern "w" option */
 static PHP_METHOD(WriteConcern, getW)
 {
+	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	const char*                wtag;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	wtag = mongoc_write_concern_get_wtag(intern->write_concern);
 
@@ -222,14 +231,18 @@ static PHP_METHOD(WriteConcern, getW)
    Returns the WriteConcern "wtimeout" option */
 static PHP_METHOD(WriteConcern, getWtimeout)
 {
+	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	int64_t                    wtimeout;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	wtimeout = mongoc_write_concern_get_wtimeout_int64(intern->write_concern);
 
@@ -246,13 +259,17 @@ static PHP_METHOD(WriteConcern, getWtimeout)
    Returns the WriteConcern "journal" option */
 static PHP_METHOD(WriteConcern, getJournal)
 {
+	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (mongoc_write_concern_journal_is_set(intern->write_concern)) {
 		RETURN_BOOL(mongoc_write_concern_get_journal(intern->write_concern));
@@ -266,18 +283,22 @@ static PHP_METHOD(WriteConcern, getJournal)
    with no write concern URI options). */
 static PHP_METHOD(WriteConcern, isDefault)
 {
+	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_BOOL(mongoc_write_concern_is_default(intern->write_concern));
 } /* }}} */
 
-static HashTable* php_phongo_write_concern_get_properties_hash(zval* object, bool is_debug, bool is_bson) /* {{{ */
+static HashTable* php_phongo_write_concern_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug, bool is_bson) /* {{{ */
 {
 	php_phongo_writeconcern_t* intern;
 	HashTable*                 props;
@@ -285,7 +306,7 @@ static HashTable* php_phongo_write_concern_get_properties_hash(zval* object, boo
 	int32_t                    w;
 	int64_t                    wtimeout;
 
-	intern = Z_WRITECONCERN_OBJ_P(object);
+	intern = Z_OBJ_WRITECONCERN(PHONGO_COMPAT_GET_OBJ(object));
 
 	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 4);
 
@@ -346,11 +367,16 @@ static HashTable* php_phongo_write_concern_get_properties_hash(zval* object, boo
 */
 static PHP_METHOD(WriteConcern, bsonSerialize)
 {
+	zend_error_handling error_handling;
+
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	ZVAL_ARR(return_value, php_phongo_write_concern_get_properties_hash(getThis(), true, true));
+	ZVAL_ARR(return_value, php_phongo_write_concern_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true, true));
 	convert_to_object(return_value);
 } /* }}} */
 
@@ -358,6 +384,7 @@ static PHP_METHOD(WriteConcern, bsonSerialize)
 */
 static PHP_METHOD(WriteConcern, serialize)
 {
+	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	zval                       retval;
 	php_serialize_data_t       var_hash;
@@ -368,9 +395,12 @@ static PHP_METHOD(WriteConcern, serialize)
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	if (!intern->write_concern) {
 		return;
@@ -417,8 +447,8 @@ static PHP_METHOD(WriteConcern, serialize)
 */
 static PHP_METHOD(WriteConcern, unserialize)
 {
-	php_phongo_writeconcern_t* intern;
 	zend_error_handling        error_handling;
+	php_phongo_writeconcern_t* intern;
 	char*                      serialized;
 	size_t                     serialized_len;
 	zval                       props;
@@ -427,7 +457,6 @@ static PHP_METHOD(WriteConcern, unserialize)
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
 		zend_restore_error_handling(&error_handling);
 		return;
@@ -519,13 +548,13 @@ static zend_object* php_phongo_writeconcern_create_object(zend_class_entry* clas
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_writeconcern_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_writeconcern_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_write_concern_get_properties_hash(object, true, false);
 } /* }}} */
 
-static HashTable* php_phongo_writeconcern_get_properties(zval* object) /* {{{ */
+static HashTable* php_phongo_writeconcern_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
 	return php_phongo_write_concern_get_properties_hash(object, false, false);
 } /* }}} */
