@@ -28,13 +28,11 @@
 
 const bson_validate_flags_t _mongoc_default_insert_vflags =
    BSON_VALIDATE_UTF8 | BSON_VALIDATE_UTF8_ALLOW_NULL |
-   BSON_VALIDATE_EMPTY_KEYS | BSON_VALIDATE_DOT_KEYS |
-   BSON_VALIDATE_DOLLAR_KEYS;
+   BSON_VALIDATE_EMPTY_KEYS;
 
 const bson_validate_flags_t _mongoc_default_replace_vflags =
    BSON_VALIDATE_UTF8 | BSON_VALIDATE_UTF8_ALLOW_NULL |
-   BSON_VALIDATE_EMPTY_KEYS | BSON_VALIDATE_DOT_KEYS |
-   BSON_VALIDATE_DOLLAR_KEYS;
+   BSON_VALIDATE_EMPTY_KEYS;
 
 const bson_validate_flags_t _mongoc_default_update_vflags =
    BSON_VALIDATE_UTF8 | BSON_VALIDATE_UTF8_ALLOW_NULL |
@@ -50,7 +48,7 @@ _mongoc_rand_simple (unsigned int *seed)
 
    err = rand_s (&ret);
    if (0 != err) {
-      MONGOC_ERROR ("rand_s failed: %");
+      MONGOC_ERROR ("rand_s failed: %s", strerror(err));
    }
 
    return (int) ret;
@@ -68,9 +66,10 @@ _mongoc_hex_md5 (const char *input)
    char digest_str[33];
    int i;
 
-   _bson_md5_init (&md5);
-   _bson_md5_append (&md5, (const uint8_t *) input, (uint32_t) strlen (input));
-   _bson_md5_finish (&md5, digest);
+   COMMON_PREFIX (_bson_md5_init (&md5));
+   COMMON_PREFIX (_bson_md5_append (
+      &md5, (const uint8_t *) input, (uint32_t) strlen (input)));
+   COMMON_PREFIX (_bson_md5_finish (&md5, digest));
 
    for (i = 0; i < sizeof digest; i++) {
       bson_snprintf (&digest_str[i * 2], 3, "%02x", digest[i]);
@@ -174,8 +173,8 @@ _mongoc_lookup_bool (const bson_t *bson, const char *key, bool default_value)
    return bson_iter_as_bool (&child);
 }
 
-void
-_mongoc_get_db_name (const char *ns, char *db /* OUT */)
+char *
+_mongoc_get_db_name (const char *ns)
 {
    size_t dblen;
    const char *dot;
@@ -185,10 +184,10 @@ _mongoc_get_db_name (const char *ns, char *db /* OUT */)
    dot = strstr (ns, ".");
 
    if (dot) {
-      dblen = BSON_MIN (dot - ns + 1, MONGOC_NAMESPACE_MAX);
-      bson_strncpy (db, ns, dblen);
+      dblen = dot - ns;
+      return bson_strndup (ns, dblen);
    } else {
-      bson_strncpy (db, ns, MONGOC_NAMESPACE_MAX);
+      return bson_strdup (ns);
    }
 }
 
@@ -323,6 +322,8 @@ _mongoc_validate_replace (const bson_t *doc,
                           bson_error_t *error)
 {
    bson_error_t validate_err;
+   bson_iter_t iter;
+   const char *key;
 
    if (vflags == BSON_VALIDATE_NONE) {
       return true;
@@ -337,6 +338,27 @@ _mongoc_validate_replace (const bson_t *doc,
       return false;
    }
 
+   if (!bson_iter_init (&iter, doc)) {
+      bson_set_error (error,
+                      MONGOC_ERROR_BSON,
+                      MONGOC_ERROR_BSON_INVALID,
+                      "replace document is corrupt");
+      return false;
+   }
+
+   while (bson_iter_next (&iter)) {
+      key = bson_iter_key (&iter);
+      if (key[0] == '$') {
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
+                         MONGOC_ERROR_COMMAND_INVALID_ARG,
+                         "Invalid key '%s': replace prohibits $ operators",
+                         key);
+
+         return false;
+      }
+   }
+
    return true;
 }
 
@@ -363,6 +385,10 @@ _mongoc_validate_update (const bson_t *update,
       return false;
    }
 
+   if (_mongoc_document_is_pipeline (update)) {
+      return true;
+   }
+
    if (!bson_iter_init (&iter, update)) {
       bson_set_error (error,
                       MONGOC_ERROR_BSON,
@@ -377,7 +403,8 @@ _mongoc_validate_update (const bson_t *update,
          bson_set_error (error,
                          MONGOC_ERROR_COMMAND,
                          MONGOC_ERROR_COMMAND_INVALID_ARG,
-                         "Invalid key '%s': update only works with $ operators",
+                         "Invalid key '%s': update only works with $ operators"
+                         " and pipelines",
                          key);
 
          return false;
@@ -521,3 +548,70 @@ _mongoc_bson_init_with_transient_txn_error (const mongoc_client_session_t *cs,
       bson_append_array_end (reply, &labels);
    }
 }
+
+bool
+_mongoc_document_is_pipeline (const bson_t *document)
+{
+   bson_iter_t iter;
+   bson_iter_t child;
+   const char *key;
+   int i = 0;
+   char *i_str;
+
+   if (!bson_iter_init (&iter, document)) {
+      return false;
+   }
+
+   while (bson_iter_next (&iter)) {
+      key = bson_iter_key (&iter);
+      i_str = bson_strdup_printf ("%d", i++);
+
+      if (strcmp (key, i_str)) {
+         bson_free (i_str);
+         return false;
+      }
+
+      bson_free (i_str);
+
+      if (BSON_ITER_HOLDS_DOCUMENT (&iter)) {
+         if (!bson_iter_recurse (&iter, &child)) {
+            return false;
+         }
+         if (!bson_iter_next (&child)) {
+            return false;
+         }
+         key = bson_iter_key (&child);
+         if (key[0] != '$') {
+            return false;
+         }
+      } else {
+         return false;
+      }
+   }
+
+   /* should return false when the document is empty */
+   return i != 0;
+}
+
+char *
+_mongoc_getenv (const char *name)
+{
+#ifdef _MSC_VER
+   char buf[2048];
+   size_t buflen;
+
+   if ((0 == getenv_s (&buflen, buf, sizeof buf, name)) && buflen) {
+      return bson_strdup (buf);
+   } else {
+      return NULL;
+   }
+#else
+
+   if (getenv (name) && strlen (getenv (name))) {
+      return bson_strdup (getenv (name));
+   } else {
+      return NULL;
+   }
+
+#endif
+}
