@@ -1,9 +1,15 @@
 <?php
 
+use MongoDB\Driver\BulkWrite;
 use MongoDB\Driver\Command;
 use MongoDB\Driver\Manager;
 use MongoDB\Driver\ReadPreference;
 use MongoDB\Driver\Server;
+use MongoDB\Driver\ServerApi;
+use MongoDB\Driver\WriteConcern;
+use MongoDB\Driver\WriteConcernError;
+use MongoDB\Driver\WriteError;
+use MongoDB\Driver\WriteResult;
 use MongoDB\Driver\Exception\ConnectionException;
 use MongoDB\Driver\Exception\RuntimeException;
 
@@ -53,7 +59,7 @@ function drop_collection($uri, $databaseName, $collectionName)
         $server->executeCommand(
             $databaseName,
             $command,
-            [ 'writeConcern' => new \MongoDB\Driver\WriteConcern( \MongoDB\Driver\WriteConcern::MAJORITY ) ]
+            ['writeConcern' => new WriteConcern(WriteConcern::MAJORITY)]
         );
     } catch (RuntimeException $e) {
         if ($e->getMessage() !== 'ns not found') {
@@ -83,6 +89,15 @@ function get_module_info($row)
     return $matches[1];
 }
 
+function create_test_manager(string $uri = null, array $options = [], array $driverOptions = [])
+{
+    if (getenv('API_VERSION') && ! isset($driverOptions['serverApi'])) {
+        $driverOptions['serverApi'] = new ServerApi(getenv('API_VERSION'));
+    }
+
+    return new Manager($uri ?? URI, $options, $driverOptions);
+}
+
 /**
  * Returns the primary server.
  *
@@ -92,7 +107,39 @@ function get_module_info($row)
  */
 function get_primary_server($uri)
 {
-    return (new Manager($uri))->selectServer(new ReadPreference('primary'));
+    return create_test_manager($uri)->selectServer(new ReadPreference('primary'));
+}
+
+/**
+ * Returns a secondary server.
+ *
+ * @param string $uri Connection string
+ * @return Server
+ * @throws ConnectionException
+ */
+function get_secondary_server($uri)
+{
+    return create_test_manager($uri)->selectServer(new ReadPreference('secondary'));
+}
+
+/**
+ * Runs a command and returns whether an exception was thrown or not
+ *
+ * @param string $uri Connection string
+ * @param array|object $commandSpec
+ * @return bool
+ * @throws RuntimeException
+ */
+function command_works($uri, $commandSpec)
+{
+    $command = new Command($commandSpec);
+    $server = get_primary_server($uri);
+    try {
+        $cursor = $server->executeCommand('admin', $command);
+        return true;
+    } catch (Exception $e) {
+        return false;
+    }
 }
 
 /**
@@ -127,6 +174,21 @@ function get_server_storage_engine($uri)
     return current($cursor->toArray())->storageEngine->name;
 }
 
+/**
+ * Helper to return the version of a specific server.
+ *
+ * @param Server $server
+ * @return string
+ * @throws RuntimeException
+ */
+function get_server_version_from_server(Server $server)
+{
+    $command = new Command(['buildInfo' => 1]);
+    $cursor = $server->executeCommand('admin', $command);
+
+    return current($cursor->toArray())->version;
+}
+
 /**
  * Returns the version of the primary server.
  *
@@ -137,10 +199,7 @@ function get_server_storage_engine($uri)
 function get_server_version($uri)
 {
     $server = get_primary_server($uri);
-    $command = new Command(['buildInfo' => 1]);
-    $cursor = $server->executeCommand('admin', $command);
-
-    return current($cursor->toArray())->version;
+    return get_server_version_from_server($server);
 }
 
 /**
@@ -171,6 +230,35 @@ function is_mongos($uri)
     return get_primary_server($uri)->getType() === Server::TYPE_MONGOS;
 }
 
+/**
+ * Checks that the topology is a sharded cluster using a replica set
+ */
+function is_mongos_with_replica_set($uri)
+{
+    if (! is_mongos($uri)) {
+        return false;
+    }
+
+    $cursor = get_primary_server($uri)->executeQuery(
+        'config.shards',
+        new \MongoDB\Driver\Query([], ['limit' => 1])
+    );
+
+    $cursor->setTypeMap(['root' => 'array', 'document' => 'array']);
+    $document = current($cursor->toArray());
+
+    if (! $document) {
+        return false;
+    }
+
+    /**
+     * Use regular expression to distinguish between standalone or replicaset:
+     * Without a replicaset: "host" : "localhost:4100"
+     * With a replicaset: "host" : "dec6d8a7-9bc1-4c0e-960c-615f860b956f/localhost:4400,localhost:4401"
+     */
+    return preg_match('@^.*/.*:\d+@', $document['host']);
+}
+
 /**
  * Checks that the topology is a replica set.
  *
@@ -179,7 +267,15 @@ function is_mongos($uri)
  */
 function is_replica_set($uri)
 {
-    return get_primary_server($uri)->getType() === Server::TYPE_RS_PRIMARY;
+    if (get_primary_server($uri)->getType() !== Server::TYPE_RS_PRIMARY) {
+        return false;
+    }
+
+    if (get_uri_option($uri, 'replicaSet') === NULL) {
+        return false;
+    }
+
+    return true;
 }
 
 /**
@@ -209,7 +305,7 @@ function is_auth($uri)
  */
 function is_ssl($uri)
 {
-    return stripos($uri, 'ssl=true') !== false;
+    return stripos($uri, 'ssl=true') !== false || stripos($uri, 'tls=true') !== false;
 }
 
 /**
@@ -337,7 +433,22 @@ function json_canonicalize($json)
      * decoding to a stdClass (see: https://bugs.php.net/bug.php?id=46600). Work
      * around this by replacing "_empty_" keys before returning.
      */
-    return str_replace('"_empty_":', '"":', $json);
+    $json = str_replace('"_empty_":', '"":', $json);
+
+    /* Canonicalize string values for $numberDouble to ensure they are converted
+     * the same as number literals in legacy and relaxed output. This is needed
+     * because the printf format in _bson_as_json_visit_double uses a high level
+     * of precision and may not produce the exponent notation expected by the
+     * BSON corpus tests. */
+    $json = preg_replace_callback(
+        '/{"\$numberDouble":"(-?\d+(\.\d+([eE]\+\d+)?)?)"}/',
+        function ($matches) {
+            return '{"$numberDouble":"' . json_encode(json_decode($matches[1])) . '"}';
+        },
+        $json
+    );
+
+    return $json;
 }
 
 /**
@@ -374,29 +485,21 @@ function NEEDS($configuration) {
         exit("skip -- need '$configuration' defined");
     }
 }
-function PREDICTABLE() {
-    global $servers;
-
-    foreach($servers as $k => $v) {
-        if (!defined($k) || !constant($k)) {
-            exit("skip - needs predictable environment (e.g. vagrant)\n");
-        }
-    }
-}
 function SLOW() {
     if (getenv("SKIP_SLOW_TESTS")) {
         exit("skip SKIP_SLOW_TESTS");
     }
 }
-function LOAD($uri, $dbname = DATABASE_NAME, $collname = COLLECTION_NAME, $filename = null) {
+
+function loadFixtures(Manager $manager, $dbname = DATABASE_NAME, $collname = COLLECTION_NAME, $filename = null)
+{
     if (!$filename) {
         $filename = "compress.zlib://" . __DIR__ . "/" . "PHONGO-FIXTURES.json.gz";
     }
 
-    $manager = new MongoDB\Driver\Manager($uri);
-    $bulk = new MongoDB\Driver\BulkWrite(['ordered' => false]);
+    $bulk = new BulkWrite(['ordered' => false]);
 
-    $server = $manager->selectServer(new MongoDB\Driver\ReadPreference(MongoDB\Driver\ReadPreference::RP_PRIMARY));
+    $server = $manager->selectServer(new ReadPreference(ReadPreference::RP_PRIMARY));
 
     $data = file_get_contents($filename);
     $array = json_decode($data);
@@ -412,11 +515,18 @@ function LOAD($uri, $dbname = DATABASE_NAME, $collname = COLLECTION_NAME, $filen
     }
 }
 
-function START($id, array $options = array()) {
-    /* starting/stopping servers only works using the Vagrant setup */
-    PREDICTABLE();
-
-    $options += array("name" => "mongod", "id" => $id);
+function createTemporaryMongoInstance(array $options = [])
+{
+    $id = 'mo_' . COLLECTION_NAME;
+    $options += [
+        "name" => "mongod",
+        "id" => $id,
+        'procParams' => [
+            'logpath' => "/tmp/MO/phongo/{$id}.log",
+            'ipv6' => true,
+            'setParameter' => [ 'enableTestCommands' => 1 ],
+        ],
+    ];
     $opts = array(
         "http" => array(
             "timeout" => 60,
@@ -428,24 +538,24 @@ function START($id, array $options = array()) {
         ),
     );
     $ctx = stream_context_create($opts);
-    $json = file_get_contents(getMOUri() . "/servers/$id", false, $ctx);
+    $json = file_get_contents(MONGO_ORCHESTRATION_URI . "/servers/$id", false, $ctx);
     $result = json_decode($json, true);
 
     /* Failed -- or was already started */
     if (!isset($result["mongodb_uri"])) {
-        DELETE($id);
-        define($id, false);
+        destroyTemporaryMongoInstance($id);
+        throw new Exception("Could not start temporary server instance\n");
     } else {
-        define($id, $result["mongodb_uri"]);
-        $FILENAME = sys_get_temp_dir() . "/PHONGO-SERVERS.json";
-
-        $json = file_get_contents($FILENAME);
-        $config = json_decode($json, true);
-        $config[$id] = constant($id);
-        file_put_contents($FILENAME, json_encode($config, JSON_PRETTY_PRINT));
+        return $result['mongodb_uri'];
     }
 }
-function DELETE($id) {
+
+function destroyTemporaryMongoInstance($id = NULL)
+{
+    if ($id == NULL) {
+        $id = 'mo_' . COLLECTION_NAME;
+    }
+
     $opts = array(
         "http" => array(
             "timeout" => 60,
@@ -455,24 +565,25 @@ function DELETE($id) {
         ),
     );
     $ctx = stream_context_create($opts);
-    $json = file_get_contents(getMOUri() . "/servers/$id", false, $ctx);
-        $FILENAME = sys_get_temp_dir() . "/PHONGO-SERVERS.json";
-
-        $json = file_get_contents($FILENAME);
-        $config = json_decode($json, true);
-        unset($config[$id]);
-        file_put_contents($FILENAME, json_encode($config, JSON_PRETTY_PRINT));
+    $json = file_get_contents(MONGO_ORCHESTRATION_URI . "/servers/$id", false, $ctx);
 }
+
 function severityToString($type) {
     switch($type) {
+    case E_DEPRECATED:
+        return "E_DEPRECATED";
+    case E_RECOVERABLE_ERROR:
+        return "E_RECOVERABLE_ERROR";
     case E_WARNING:
         return "E_WARNING";
+    case E_NOTICE:
+        return "E_NOTICE";
     default:
         return "Some other #_$type";
     }
 }
 function raises($function, $type, $infunction = null) {
-    $errhandler = function($severity, $message, $file, $line, $errcontext) {
+    $errhandler = function($severity, $message, $file, $line) {
         throw new ErrorException($message, 0, $severity, $file, $line);
     };
 
@@ -480,6 +591,8 @@ function raises($function, $type, $infunction = null) {
     try {
         $function();
     } catch(Exception $e) {
+        $exceptionname = get_class($e);
+
         if ($e instanceof ErrorException && $e->getSeverity() & $type) {
             if ($infunction) {
                 $trace = $e->getTrace();
@@ -492,7 +605,7 @@ function raises($function, $type, $infunction = null) {
                 restore_error_handler();
                 return $e->getMessage();
             }
-            printf("OK: Got %s\n", severityToString($type));
+            printf("OK: Got %s\n", severityToString($e->getSeverity()));
         } else {
             printf("ALMOST: Got %s - expected %s\n", get_class($e), $exceptionname);
         }
@@ -500,39 +613,47 @@ function raises($function, $type, $infunction = null) {
         return $e->getMessage();
     }
 
-    echo "FAILED: Expected $exceptionname thrown!\n";
+    printf("FAILED: Expected %s thrown!\n", ErrorException::class);
     restore_error_handler();
 }
 function throws($function, $exceptionname, $infunction = null) {
     try {
         $function();
-    } catch(Exception $e) {
-        $message = str_replace(array("\n", "\r"), ' ', $e->getMessage());
-        if ($e instanceof $exceptionname) {
-            if ($infunction) {
-                $trace = $e->getTrace();
-                $function = $trace[0]["function"];
-                if (strcasecmp($function, $infunction) == 0) {
-                    printf("OK: Got %s thrown from %s\n", $exceptionname, $infunction);
-                } else {
-                    printf("ALMOST: Got %s - but was thrown in %s, not %s (%s)\n", $exceptionname, $function, $infunction, $message);
-                }
-                return $e->getMessage();
-            }
-            printf("OK: Got %s\n", $exceptionname);
-        } else {
-            printf("ALMOST: Got %s (%s) - expected %s\n", get_class($e), $message, $exceptionname);
-        }
-        return $e->getMessage();
+    } catch (Throwable $e) {
+    } catch (Exception $e) {
     }
-    echo "FAILED: Expected $exceptionname thrown, but no exception thrown!\n";
+
+    if (!isset($e)) {
+        echo "FAILED: Expected $exceptionname thrown, but no exception thrown!\n";
+        return;
+    }
+
+    $message = str_replace(array("\n", "\r"), ' ', $e->getMessage());
+    if ($e instanceof $exceptionname) {
+        if ($infunction) {
+            $trace = $e->getTrace();
+            $function = $trace[0]["function"];
+            if (strcasecmp($function, $infunction) == 0) {
+                printf("OK: Got %s thrown from %s\n", $exceptionname, $infunction);
+            } else {
+                printf("ALMOST: Got %s - but was thrown in %s, not %s (%s)\n", $exceptionname, $function, $infunction, $message);
+            }
+            return $e->getMessage();
+        }
+        printf("OK: Got %s\n", $exceptionname);
+    } else {
+        printf("ALMOST: Got %s (%s) - expected %s\n", get_class($e), $message, $exceptionname);
+    }
+
+    return $e->getMessage();
 }
-function printServer(MongoDB\Driver\Server $server)
+
+function printServer(Server $server)
 {
     printf("server: %s:%d\n", $server->getHost(), $server->getPort());
 }
 
-function printWriteResult(MongoDB\Driver\WriteResult $result, $details = true)
+function printWriteResult(WriteResult $result, $details = true)
 {
     printServer($result->getServer());
 
@@ -555,7 +676,7 @@ function printWriteResult(MongoDB\Driver\WriteResult $result, $details = true)
     }
 }
 
-function printWriteConcernError(MongoDB\Driver\WriteConcernError $error = null, $details)
+function printWriteConcernError(WriteConcernError $error = null, $details)
 {
     if ($error) {
         /* This stuff is generated by the server, no need for us to test it */
@@ -571,7 +692,7 @@ function printWriteConcernError(MongoDB\Driver\WriteConcernError $error = null,
     }
 }
 
-function printWriteError(MongoDB\Driver\WriteError $error)
+function printWriteError(WriteError $error)
 {
     var_dump($error);
     printf("writeError[%d].message: %s\n", $error->getIndex(), $error->getMessage());
@@ -600,49 +721,37 @@ function def($arr) {
     }
 }
 
-function configureFailPoint(MongoDB\Driver\Manager $manager, $failPoint, $mode, $data = array()) {
+function configureFailPoint(Manager $manager, $failPoint, $mode, array $data = [])
+{
+    $doc = [
+        'configureFailPoint' => $failPoint,
+        'mode'               => $mode,
+    ];
+    if ($data) {
+        $doc['data'] = $data;
+    }
 
+    $cmd = new Command($doc);
+    $manager->executeCommand('admin', $cmd);
+}
+
+function configureTargetedFailPoint(Server $server, $failPoint, $mode, array $data = [])
+{
     $doc = array(
-        "configureFailPoint" => $failPoint,
-        "mode"               => $mode,
+        'configureFailPoint' => $failPoint,
+        'mode'               => $mode,
     );
     if ($data) {
-        $doc["data"] = $data;
+        $doc['data'] = $data;
     }
 
-    $cmd = new MongoDB\Driver\Command($doc);
-    $result = $manager->executeCommand("admin", $cmd);
-    $arr = current($result->toArray());
-    if (empty($arr->ok)) {
-        var_dump($result);
-        throw new RuntimeException("Failpoint failed");
-    }
-    return true;
+    $cmd = new Command($doc);
+    $server->executeCommand('admin', $cmd);
 }
 
-function failMaxTimeMS(MongoDB\Driver\Manager $manager) {
-    return configureFailPoint($manager, "maxTimeAlwaysTimeOut", array("times" => 1));
-}
-
-function getMOUri() {
-    if (!($HOST = getenv("MONGODB_ORCHESTRATION_HOST"))) {
-        $HOST = "192.168.112.10";
-    }
-
-    if (!($PORT = getenv("MONGODB_ORCHESTRATION_PORT"))) {
-        $PORT = "8889";
-    }
-    $MO = "http://$HOST:$PORT/v1";
-
-    return $MO;
-}
-
-function getMOPresetBase() {
-    if (!($BASE = getenv("mongodb_orchestration_base"))) {
-        $BASE = "/phongo/";
-    }
-
-    return $BASE;
+function failMaxTimeMS(Server $server)
+{
+    configureTargetedFailPoint($server, 'maxTimeAlwaysTimeOut', [ 'times' => 1 ]);
 }
 
 function toPHP($var, $typemap = array()) {
@@ -666,6 +775,26 @@ function fromJSON($var) {
 
 /* Note: this fail point may terminate the mongod process, so you may want to
  * use this in conjunction with a throwaway server. */
-function failGetMore(MongoDB\Driver\Manager $manager) {
-    return configureFailPoint($manager, "failReceivedGetmore", "alwaysOn");
+function failGetMore(Manager $manager)
+{
+    /* We need to do version detection here */
+    $primary = $manager->selectServer(new ReadPreference('primary'));
+    $version = get_server_version_from_server($primary);
+
+    if (version_compare($version, "3.2", "<")) {
+        configureFailPoint($manager, 'failReceivedGetmore', 'alwaysOn');
+        return;
+    }
+
+    if (version_compare($version, "4.0", ">=")) {
+        /* We use 237 here, as that's the same original code that MongoD would
+         * throw if a cursor had already gone by the time we call getMore. This
+         * allows us to make things consistent with the getMore OP behaviour
+         * from previous mongod versions. An errorCode is required here for the
+         * failPoint to work. */
+        configureFailPoint($manager, 'failCommand', 'alwaysOn', [ 'errorCode' => 237, 'failCommands' => ['getMore'] ]);
+        return;
+    }
+
+    throw new Exception("Trying to configure a getMore fail point for a server version ($version) that doesn't support it");
 }
